@@ -5,6 +5,15 @@
 
 #include <osmo-bts/gsm_data.h>
 
+/* Whether a logical channel must be activated automatically */
+#define TRX_CHAN_FLAG_AUTO_ACTIVE	(1 << 0)
+/* Whether a logical channel belongs to PDCH (packet switched data) */
+#define TRX_CHAN_FLAG_PDCH		(1 << 1)
+
+/* FIXME: we should actually activate 'auto-active' channels */
+#define TRX_CHAN_IS_ACTIVE(state, chan) \
+	(trx_chan_desc[chan].flags & TRX_CHAN_FLAG_AUTO_ACTIVE || (state)->active)
+
 /* These types define the different channels on a multiframe.
  * Each channel has queues and can be activated individually.
  */
@@ -47,11 +56,10 @@ enum trx_chan_type {
 	TRXC_SACCH8_7,
 	TRXC_PDTCH,
 	TRXC_PTCCH,
+	TRXC_CBCH,
 	_TRX_CHAN_MAX
 };
 
-extern const struct value_string trx_chan_type_names[];
-
 #define GSM_BURST_LEN		148
 #define GPRS_BURST_LEN		GSM_BURST_LEN
 #define EGPRS_BURST_LEN		444
@@ -71,14 +79,19 @@ struct l1sched_chan_state {
 	uint32_t		ul_first_fn;	/* fn of first burst */
 	uint8_t			ul_mask;	/* mask of received bursts */
 
-	/* RSSI / TOA */
+	/* measurements */
 	uint8_t			rssi_num;	/* number of RSSI values */
 	float			rssi_sum;	/* sum of RSSI values */
 	uint8_t			toa_num;	/* number of TOA values */
 	int32_t			toa256_sum;	/* sum of TOA values (1/256 symbol) */
+	uint8_t			ci_cb_num;	/* number of C/I values */
+	int32_t			ci_cb_sum;	/* sum of C/I values (in centiBels) */
 
 	/* loss detection */
-	uint8_t			lost;		/* (SACCH) loss detection */
+	uint8_t			lost_frames;	/* how many L2 frames were lost */
+	uint32_t		last_tdma_fn;	/* last processed TDMA frame number */
+	uint32_t		proc_tdma_fs;	/* how many TDMA frames were processed */
+	uint32_t		lost_tdma_fs;	/* how many TDMA frames were lost */
 
 	/* mode */
 	uint8_t			rsl_cmode, tch_mode; /* mode for TCH channels */
@@ -108,22 +121,16 @@ struct l1sched_chan_state {
 
 	/* measurements */
 	struct {
-		uint8_t		clock;		/* cyclic clock counter */
-		int8_t		rssi[32];	/* last RSSI values */
-		int		rssi_count;	/* received RSSI values */
-		int		rssi_valid_count; /* number of stored value */
-		int		rssi_got_burst; /* any burst received so far */
 		int32_t		toa256_sum;	/* sum of TOA values (1/256 symbol) */
 		int		toa_num;	/* number of TOA value */
 	} meas;
 
 	/* handover */
-	uint8_t			ho_rach_detect;	/* if rach detection is on */
+	bool			ho_rach_detect;	/* if rach detection is on */
 };
 
 struct l1sched_ts {
 	uint8_t 		mf_index;	/* selected multiframe index */
-	uint32_t 		mf_last_fn;	/* last received frame number */
 	uint8_t			mf_period;	/* period of multiframe */
 	const struct trx_sched_frame *mf_frames; /* pointer to frame layout */
 
@@ -163,9 +170,11 @@ int trx_sched_tch_req(struct l1sched_trx *l1t, struct osmo_phsap_prim *l1sap);
 /*! \brief PHY informs us of new (current) GSM frame number */
 int trx_sched_clock(struct gsm_bts *bts, uint32_t fn);
 
-/*! \brief handle an UL burst received by PHY */
-int trx_sched_ul_burst(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-        sbit_t *bits, uint16_t nbits, int8_t rssi, int16_t toa);
+/*! \brief PHY informs us clock indications should start to be received */
+int trx_sched_clock_started(struct gsm_bts *bts);
+
+/*! \brief PHY informs us no more clock indications should be received anymore */
+int trx_sched_clock_stopped(struct gsm_bts *bts);
 
 /*! \brief set multiframe scheduler to given physical channel config */
 int trx_sched_set_pchan(struct l1sched_trx *l1t, uint8_t tn,
@@ -221,4 +230,34 @@ int find_sched_mframe_idx(enum gsm_phys_chan_config pchan, uint8_t tn);
 bool trx_sched_is_sacch_fn(struct gsm_bts_trx_ts *ts, uint32_t fn, bool uplink);
 extern const struct trx_sched_multiframe trx_sched_multiframes[];
 
+#define TRX_BI_F_NOPE_IND	(1 << 0)
+#define TRX_BI_F_MOD_TYPE	(1 << 1)
+#define TRX_BI_F_TS_INFO	(1 << 2)
+#define TRX_BI_F_CI_CB		(1 << 3)
+
+/*! UL burst indication with the corresponding meta info */
+struct trx_ul_burst_ind {
+	/* Field presence bitmask (see TRX_BI_F_*) */
+	uint8_t flags;
+
+	/* Mandatory fields */
+	uint32_t fn;		/*!< TDMA frame number */
+	uint8_t tn;		/*!< TDMA time-slot number */
+	int16_t toa256;		/*!< Timing of Arrival in units of 1/256 of symbol */
+	int8_t rssi;		/*!< Received Signal Strength Indication */
+
+	/* Optional fields (defined by flags) */
+	enum trx_burst_type bt;	/*!< Modulation type */
+	uint8_t tsc_set;	/*!< Training Sequence Set */
+	uint8_t tsc;		/*!< Training Sequence Code */
+	int16_t ci_cb;		/*!< Carrier-to-Interference ratio (in centiBels) */
+
+	/*! Burst soft-bits buffer */
+	sbit_t burst[EGPRS_BURST_LEN];
+	size_t burst_len;
+};
+
+/*! Handle an UL burst received by PHY */
+int trx_sched_ul_burst(struct l1sched_trx *l1t, struct trx_ul_burst_ind *bi);
+
 #endif /* TRX_SCHEDULER_H */
