@@ -13,37 +13,35 @@ typedef ubit_t *trx_sched_dl_func(struct l1sched_trx *l1t, uint8_t tn,
 				  uint32_t fn, enum trx_chan_type chan,
 				  uint8_t bid, uint16_t *nbits);
 
-typedef int trx_sched_ul_func(struct l1sched_trx *l1t, uint8_t tn,
-			      uint32_t fn, enum trx_chan_type chan,
-			      uint8_t bid, sbit_t *bits, uint16_t nbits,
-			      int8_t rssi, int16_t toa256);
+typedef int trx_sched_ul_func(struct l1sched_trx *l1t, enum trx_chan_type chan,
+			      uint8_t bid, const struct trx_ul_burst_ind *bi);
 
 struct trx_chan_desc {
-	/*! \brief Is this on a PDCH (PS) ? */
-	int			pdch;
-	/*! \brief TRX Channel Type */
-	enum trx_chan_type	chan;
+	/*! \brief Human-readable name */
+	const char		*name;
+	/*! \brief Human-readable description */
+	const char		*desc;
 	/*! \brief Channel Number (like in RSL) */
 	uint8_t			chan_nr;
 	/*! \brief Link ID (like in RSL) */
 	uint8_t			link_id;
-	/*! \brief Human-readable name */
-	const char		*name;
 	/*! \brief function to call when we want to generate RTS.req to L2 */
 	trx_sched_rts_func	*rts_fn;
 	/*! \brief function to call when DATA.req received from L2 */
 	trx_sched_dl_func	*dl_fn;
 	/*! \brief function to call when burst received from PHY */
 	trx_sched_ul_func	*ul_fn;
-	/*! \brief is this channel automatically active at start? */
-	int			auto_active;
+	/*! \brief function to call when NOPE.ind received from PHY */
+	trx_sched_ul_func	*nope_fn;
+	/*! \brief channel flags, see TRX_CHAN_FLAG_* */
+	uint8_t			flags;
 };
 extern const struct trx_chan_desc trx_chan_desc[_TRX_CHAN_MAX];
 
 extern const ubit_t _sched_tsc[8][26];
 extern const ubit_t _sched_egprs_tsc[8][78];
-const ubit_t _sched_fcch_burst[148];
-const ubit_t _sched_sch_train[64];
+extern const ubit_t _sched_fcch_burst[148];
+extern const ubit_t _sched_sch_train[64];
 
 struct msgb *_sched_dequeue_prim(struct l1sched_trx *l1t, int8_t tn, uint32_t fn,
 				 enum trx_chan_type chan);
@@ -72,21 +70,16 @@ ubit_t *tx_tchf_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	enum trx_chan_type chan, uint8_t bid, uint16_t *nbits);
 ubit_t *tx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	enum trx_chan_type chan, uint8_t bid, uint16_t *nbits);
-int rx_rach_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, int16_t toa256);
-int rx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, int16_t toa256);
-int rx_pdtch_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, int16_t toa256);
-int rx_tchf_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, int16_t toa256);
-int rx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, int16_t toa256);
+int rx_rach_fn(struct l1sched_trx *l1t, enum trx_chan_type chan,
+	       uint8_t bid, const struct trx_ul_burst_ind *bi);
+int rx_data_fn(struct l1sched_trx *l1t, enum trx_chan_type chan,
+	       uint8_t bid, const struct trx_ul_burst_ind *bi);
+int rx_pdtch_fn(struct l1sched_trx *l1t, enum trx_chan_type chan,
+	        uint8_t bid, const struct trx_ul_burst_ind *bi);
+int rx_tchf_fn(struct l1sched_trx *l1t, enum trx_chan_type chan,
+	       uint8_t bid, const struct trx_ul_burst_ind *bi);
+int rx_tchh_fn(struct l1sched_trx *l1t, enum trx_chan_type chan,
+	       uint8_t bid, const struct trx_ul_burst_ind *bi);
 
 const ubit_t *_sched_dl_burst(struct l1sched_trx *l1t, uint8_t tn,
 			      uint32_t fn, uint16_t *nbits);
