@@ -31,6 +31,7 @@
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
+#include <inttypes.h>
 
 #include <osmocom/core/select.h>
 #include <osmocom/core/timer.h>
@@ -107,9 +108,11 @@ static struct e1inp_sign_link *sign_link_up(void *unit, struct e1inp_line *line,
 		e1inp_ts_config_sign(&line->ts[E1INP_SIGN_OML-1], line);
 		sign_link = g_bts->oml_link =
 			e1inp_sign_link_create(&line->ts[E1INP_SIGN_OML-1],
-						E1INP_SIGN_OML, NULL, 255, 0);
+						E1INP_SIGN_OML, g_bts->c0, 255, 0);
+		if (clock_gettime(CLOCK_MONOTONIC, &g_bts->oml_conn_established_timestamp) != 0)
+			memset(&g_bts->oml_conn_established_timestamp, 0,
+			       sizeof(g_bts->oml_conn_established_timestamp));
 		drain_oml_queue(g_bts);
-		sign_link->trx = g_bts->c0;
 		bts_link_estab(g_bts);
 		break;
 	default:
@@ -118,15 +121,14 @@ static struct e1inp_sign_link *sign_link_up(void *unit, struct e1inp_line *line,
 			trx_nr);
 		trx = gsm_bts_trx_num(g_bts, trx_nr);
 		if (!trx) {
-			LOGP(DABIS, LOGL_ERROR, "TRX%d does not exixt!\n",
+			LOGP(DABIS, LOGL_ERROR, "TRX%d does not exist!\n",
 				trx_nr);
 			break;
 		}
 		e1inp_ts_config_sign(&line->ts[type-1], line);
 		sign_link = trx->rsl_link =
 			e1inp_sign_link_create(&line->ts[type-1],
-						E1INP_SIGN_RSL, NULL, 0, 0);
-		sign_link->trx = trx;
+						E1INP_SIGN_RSL, trx, 0, 0);
 		trx_link_estab(trx);
 		break;
 	}
@@ -140,9 +142,22 @@ static void sign_link_down(struct e1inp_line *line)
 	LOGP(DABIS, LOGL_ERROR, "Signalling link down\n");
 
 	/* First remove the OML signalling link */
-	if (g_bts->oml_link)
+	if (g_bts->oml_link) {
+		struct timespec now;
+
 		e1inp_sign_link_destroy(g_bts->oml_link);
+
+		/* Log a special notice if the OML connection was dropped relatively quickly. */
+		if (g_bts->oml_conn_established_timestamp.tv_sec != 0 && clock_gettime(CLOCK_MONOTONIC, &now) == 0 &&
+		    g_bts->oml_conn_established_timestamp.tv_sec + OSMO_BTS_OML_CONN_EARLY_DISCONNECT >= now.tv_sec) {
+			LOGP(DABIS, LOGL_FATAL, "OML link was closed early within %" PRIu64 " seconds. "
+			"If this situation persists, please check your BTS and BSC configuration files for errors. "
+			"A common error is a mismatch between unit_id configuration parameters of BTS and BSC.\n",
+			(uint64_t)(now.tv_sec - g_bts->oml_conn_established_timestamp.tv_sec));
+		}
+	}
 	g_bts->oml_link = NULL;
+	memset(&g_bts->oml_conn_established_timestamp, 0, sizeof(g_bts->oml_conn_established_timestamp));
 
 	/* Then iterate over the RSL signalling links */
 	llist_for_each_entry(trx, &g_bts->trx_list, list) {
@@ -156,7 +171,7 @@ static void sign_link_down(struct e1inp_line *line)
 }
 
 
-/* callback for incoming mesages from A-bis/IP */
+/* callback for incoming messages from A-bis/IP */
 static int sign_link_cb(struct msgb *msg)
 {
 	struct e1inp_sign_link *link = msg->dst;
@@ -195,7 +210,7 @@ uint32_t get_signlink_remote_ip(struct e1inp_sign_link *link)
 		return 0;
 	}
 
-	/* we assume that the soket is AF_INET.  As Abis/IP contains
+	/* we assume that the socket is AF_INET.  As Abis/IP contains
 	 * lots of hard-coded IPv4 addresses, this safe */
 	OSMO_ASSERT(sin.sin_family == AF_INET);
 
@@ -209,7 +224,9 @@ static int inp_s_cbfn(unsigned int subsys, unsigned int signal,
 	if (subsys != SS_L_INPUT)
 		return 0;
 
-	DEBUGP(DABIS, "Input Signal %u received\n", signal);
+	struct input_signal_data *isd = signal_data;
+	DEBUGP(DABIS, "Input Signal %s received for link_type=%s\n",
+	       get_value_string(e1inp_signal_names, signal), e1inp_signtype_name(isd->link_type));
 
 	return 0;
 }
@@ -241,7 +258,7 @@ void abis_init(struct gsm_bts *bts)
 	g_bts = bts;
 
 	oml_init(&bts->mo);
-	libosmo_abis_init(NULL);
+	libosmo_abis_init(tall_bts_ctx);
 
 	osmo_signal_register_handler(SS_L_INPUT, &inp_s_cbfn, bts);
 }
@@ -251,7 +268,7 @@ struct e1inp_line *abis_open(struct gsm_bts *bts, char *dst_host,
 {
 	struct e1inp_line *line;
 
-	/* patch in various data from VTY and othe sources */
+	/* patch in various data from VTY and other sources */
 	line_ops.cfg.ipa.addr = dst_host;
 	osmo_get_macaddr(bts_dev_info.mac_addr, "eth0");
 	bts_dev_info.site_id = bts->ip_access.site_id;
