@@ -32,6 +32,7 @@
 #include <osmocom/core/msgb.h>
 #include <osmocom/gsm/l1sap.h>
 #include <osmocom/gsm/gsm_utils.h>
+#include <osmocom/gsm/rsl.h>
 #include <osmocom/core/gsmtap.h>
 #include <osmocom/core/gsmtap_util.h>
 #include <osmocom/core/utils.h>
@@ -53,11 +54,58 @@
 #include <osmo-bts/power_control.h>
 #include <osmo-bts/msg_utils.h>
 #include <osmo-bts/pcuif_proto.h>
+#include <osmo-bts/cbch.h>
+
+
+#define CB_FCCH		-1
+#define CB_SCH		-2
+#define CB_BCCH		-3
+#define CB_IDLE		-4
+
+/* according to TS 05.02 Clause 7 Table 3 of 9 an Figure 8a */
+static const int ccch_block_table[51] = {
+	CB_FCCH, CB_SCH,/* 0..1 */
+	CB_BCCH, CB_BCCH, CB_BCCH, CB_BCCH,	/* 2..5: BCCH */
+	0, 0, 0, 0,	/* 6..9: B0 */
+	CB_FCCH, CB_SCH,/* 10..11 */
+	1, 1, 1, 1,	/* 12..15: B1 */
+	2, 2, 2, 2,	/* 16..19: B2 */
+	CB_FCCH, CB_SCH,/* 20..21 */
+	3, 3, 3, 3,	/* 22..25: B3 */
+	4, 4, 4, 4,	/* 26..29: B4 */
+	CB_FCCH, CB_SCH,/* 30..31 */
+	5, 5, 5, 5,	/* 32..35: B5 */
+	6, 6, 6, 6,	/* 36..39: B6 */
+	CB_FCCH, CB_SCH,/* 40..41 */
+	7, 7, 7, 7, 	/* 42..45: B7 */
+	8, 8, 8, 8, 	/* 46..49: B8 */
+	-4		/* 50: Idle */
+};
+
+/* determine the CCCH block number based on the frame number */
+unsigned int l1sap_fn2ccch_block(uint32_t fn)
+{
+	int rc = ccch_block_table[fn%51];
+	/* if FN is negative, we were called for something that's not CCCH! */
+	OSMO_ASSERT(rc >= 0);
+	return rc;
+}
 
 struct gsm_lchan *get_lchan_by_chan_nr(struct gsm_bts_trx *trx,
 				       unsigned int chan_nr)
 {
-	return &trx->ts[L1SAP_CHAN2TS(chan_nr)].lchan[l1sap_chan2ss(chan_nr)];
+	unsigned int tn, ss;
+
+	tn = L1SAP_CHAN2TS(chan_nr);
+	OSMO_ASSERT(tn < ARRAY_SIZE(trx->ts));
+
+	if (L1SAP_IS_CHAN_CBCH(chan_nr))
+		ss = 2; /* CBCH is always on sub-slot 2 */
+	else
+		ss = l1sap_chan2ss(chan_nr);
+	OSMO_ASSERT(ss < ARRAY_SIZE(trx->ts[tn].lchan));
+
+	return &trx->ts[tn].lchan[ss];
 }
 
 static struct gsm_lchan *
@@ -66,9 +114,8 @@ get_active_lchan_by_chan_nr(struct gsm_bts_trx *trx, unsigned int chan_nr)
 	struct gsm_lchan *lchan = get_lchan_by_chan_nr(trx, chan_nr);
 
 	if (lchan && lchan->state != LCHAN_S_ACTIVE) {
-		LOGP(DL1P, LOGL_NOTICE, "%s: assuming active lchan, but "
-		     "state is %s\n", gsm_lchan_name(lchan),
-		     gsm_lchans_name(lchan->state));
+		LOGPLCHAN(lchan, DL1P, LOGL_NOTICE, "assuming active lchan, but state is %s\n",
+			  gsm_lchans_name(lchan->state));
 		return NULL;
 	}
 	return lchan;
@@ -91,9 +138,9 @@ static uint32_t fn_ms_adj(uint32_t fn, const struct gsm_lchan *lchan)
 		r -= r % GSM_RTP_DURATION;
 
 		if (r != GSM_RTP_DURATION)
-			LOGP(DRTP, LOGL_ERROR, "RTP clock out of sync with lower layer:"
-				" %"PRIu32" vs %d (%"PRIu32"->%"PRIu32")\n",
-				r, GSM_RTP_DURATION, lchan->tch.last_fn, fn);
+			LOGPLCHAN(lchan, DRTP, LOGL_ERROR, "RTP clock out of sync with lower layer:"
+				  " %"PRIu32" vs %d (%"PRIu32"->%"PRIu32")\n",
+				  r, GSM_RTP_DURATION, lchan->tch.last_fn, fn);
 	}
 	return GSM_RTP_DURATION;
 }
@@ -101,7 +148,7 @@ static uint32_t fn_ms_adj(uint32_t fn, const struct gsm_lchan *lchan)
 /*! limit number of queue entries to %u; drops any surplus messages */
 static void queue_limit_to(const char *prefix, struct llist_head *queue, unsigned int limit)
 {
-	int count = llist_count(queue);
+	unsigned int count = llist_count(queue);
 
 	if (count > limit)
 		LOGP(DL1P, LOGL_NOTICE, "%s: freeing %d queued frames\n", prefix, count-limit);
@@ -113,7 +160,7 @@ static void queue_limit_to(const char *prefix, struct llist_head *queue, unsigne
 }
 
 /* allocate a msgb containing a osmo_phsap_prim + optional l2 data
- * in order to wrap femtobts header arround l2 data, there must be enough space
+ * in order to wrap femtobts header around l2 data, there must be enough space
  * in front and behind data pointer */
 struct msgb *l1sap_msgb_alloc(unsigned int l2_len)
 {
@@ -135,12 +182,10 @@ int add_l1sap_header(struct gsm_bts_trx *trx, struct msgb *rmsg,
 {
 	struct osmo_phsap_prim *l1sap;
 
-	LOGP(DL1P, LOGL_DEBUG, "%s Rx -> RTP: %s\n",
-	     gsm_lchan_name(lchan), osmo_hexdump(rmsg->data, rmsg->len));
+	LOGPLCHAN(lchan, DL1P, LOGL_DEBUG, "Rx -> RTP: %s\n", osmo_hexdump(rmsg->data, rmsg->len));
 
 	rmsg->l2h = rmsg->data;
-	msgb_push(rmsg, sizeof(*l1sap));
-	rmsg->l1h = rmsg->data;
+	rmsg->l1h = msgb_push(rmsg, sizeof(*l1sap));
 	l1sap = msgb_l1sap_prim(rmsg);
 	osmo_prim_init(&l1sap->oph, SAP_GSM_PH, PRIM_TCH, PRIM_OP_INDICATION,
 		       rmsg);
@@ -212,6 +257,85 @@ int bts_check_for_ciph_cmd(struct msgb *msg, struct gsm_lchan *lchan,
 	return check_for_ciph_cmd(msg, lchan, chan_nr);
 }
 
+uint16_t l1sap_log_ctx_sapi;
+
+const struct value_string l1sap_common_sapi_names[] = {
+	{ L1SAP_COMMON_SAPI_UNKNOWN,	"UNKNOWN" },
+	/* alphabetic order */
+	{ L1SAP_COMMON_SAPI_AGCH,	"AGCH" },
+	{ L1SAP_COMMON_SAPI_BCCH,	"BCCH" },
+	{ L1SAP_COMMON_SAPI_CBCH,	"CBCH" },
+	{ L1SAP_COMMON_SAPI_FACCH_F,	"FACCH/F" },
+	{ L1SAP_COMMON_SAPI_FACCH_H,	"FACCH/H" },
+	{ L1SAP_COMMON_SAPI_FCCH,	"FCCH" },
+	{ L1SAP_COMMON_SAPI_IDLE,	"IDLE" },
+	{ L1SAP_COMMON_SAPI_NCH,	"NCH" },
+	{ L1SAP_COMMON_SAPI_PACCH,	"PACCH" },
+	{ L1SAP_COMMON_SAPI_PAGCH,	"PAGCH" },
+	{ L1SAP_COMMON_SAPI_PBCCH,	"PBCCH" },
+	{ L1SAP_COMMON_SAPI_PCH,	"PCH" },
+	{ L1SAP_COMMON_SAPI_PDTCH,	"PDTCH" },
+	{ L1SAP_COMMON_SAPI_PNCH,	"PNCH" },
+	{ L1SAP_COMMON_SAPI_PPCH,	"PPCH" },
+	{ L1SAP_COMMON_SAPI_PRACH,	"PRACH" },
+	{ L1SAP_COMMON_SAPI_PTCCH,	"PTCCH" },
+	{ L1SAP_COMMON_SAPI_RACH,	"RACH" },
+	{ L1SAP_COMMON_SAPI_SACCH,	"SACCH" },
+	{ L1SAP_COMMON_SAPI_SCH,	"SCH" },
+	{ L1SAP_COMMON_SAPI_SDCCH,	"SDCCH" },
+	{ L1SAP_COMMON_SAPI_TCH_F,	"TCH/F" },
+	{ L1SAP_COMMON_SAPI_TCH_H,	"TCH/H" },
+	{ 0, NULL }
+};
+
+static enum l1sap_common_sapi get_common_sapi_ph_data(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
+{
+	uint8_t link_id = l1sap->u.data.link_id;
+	uint8_t chan_nr = l1sap->u.data.chan_nr;
+	uint32_t u32Fn = l1sap->u.data.fn;
+
+	if (L1SAP_IS_CHAN_TCHF(chan_nr))
+		return L1SAP_COMMON_SAPI_TCH_F;
+
+	if (L1SAP_IS_CHAN_TCHH(chan_nr))
+		return L1SAP_COMMON_SAPI_TCH_H;
+
+	if (L1SAP_IS_CHAN_SDCCH4(chan_nr) || L1SAP_IS_CHAN_SDCCH8(chan_nr))
+		return L1SAP_COMMON_SAPI_SDCCH;
+
+	if (L1SAP_IS_CHAN_BCCH(chan_nr))
+		return L1SAP_COMMON_SAPI_BCCH;
+
+	if (L1SAP_IS_CHAN_AGCH_PCH(chan_nr))
+		/* The sapi depends on DSP configuration, not on the actual SYSTEM INFORMATION 3. */
+		return ((l1sap_fn2ccch_block(u32Fn) >= num_agch(trx, "PH-DATA-REQ"))
+			? L1SAP_COMMON_SAPI_PCH
+			: L1SAP_COMMON_SAPI_AGCH);
+
+	if (L1SAP_IS_CHAN_CBCH(chan_nr))
+		return L1SAP_COMMON_SAPI_CBCH;
+
+	if (L1SAP_IS_LINK_SACCH(link_id))
+		return L1SAP_COMMON_SAPI_SACCH;
+
+	return L1SAP_COMMON_SAPI_UNKNOWN;
+}
+
+static enum l1sap_common_sapi get_common_sapi_by_trx_prim(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
+{
+	/* Only downlink prims are relevant */
+	switch (OSMO_PRIM_HDR(&l1sap->oph)) {
+	case OSMO_PRIM(PRIM_PH_DATA, PRIM_OP_REQUEST):
+		if (ts_is_pdch(&trx->ts[L1SAP_CHAN2TS(l1sap->u.data.chan_nr)]))
+			return ((L1SAP_IS_PTCCH(l1sap->u.data.fn))
+				? L1SAP_COMMON_SAPI_PTCCH
+				: L1SAP_COMMON_SAPI_PDTCH);
+		return get_common_sapi_ph_data(trx, l1sap);
+	default:
+		return L1SAP_COMMON_SAPI_UNKNOWN;
+	}
+}
+
 struct gsmtap_inst *gsmtap = NULL;
 uint32_t gsmtap_sapi_mask = 0;
 uint8_t gsmtap_sapi_acch = 0;
@@ -262,10 +386,12 @@ static int gsmtap_ph_data(struct osmo_phsap_prim *l1sap, uint8_t *chan_type,
 	} else if (L1SAP_IS_CHAN_AGCH_PCH(chan_nr)) {
 		/* The sapi depends on DSP configuration, not
 		 * on the actual SYSTEM INFORMATION 3. */
-		if (L1SAP_FN2CCCHBLOCK(fn) >= num_agch)
+		if (l1sap_fn2ccch_block(fn) >= num_agch)
 			*chan_type = GSMTAP_CHANNEL_PCH;
 		else
 			*chan_type = GSMTAP_CHANNEL_AGCH;
+	} else if (L1SAP_IS_CHAN_CBCH(chan_nr)) {
+		*chan_type = GSMTAP_CHANNEL_CBCH51;
 	} else if (L1SAP_IS_CHAN_PDCH(chan_nr)) {
 		*chan_type = GSMTAP_CHANNEL_PDTCH;
 	}
@@ -286,15 +412,10 @@ static int gsmtap_pdch(struct osmo_phsap_prim *l1sap, uint8_t *chan_type,
 	if (L1SAP_IS_PTCCH(fn)) {
 		*chan_type = GSMTAP_CHANNEL_PTCCH;
 		*ss = L1SAP_FN2PTCCHBLOCK(fn);
-		if (l1sap->oph.primitive == PRIM_OP_INDICATION) {
-			OSMO_ASSERT(len > 0);
-			if ((*data[0]) == 7)
-				return -EINVAL;
-			(*data)++;
-			(*len)--;
-		}
-	} else
-		*chan_type = GSMTAP_CHANNEL_PACCH;
+	} else {
+		/* TODO: distinguish PACCH */
+		*chan_type = GSMTAP_CHANNEL_PDTCH;
+	}
 
 	return 0;
 }
@@ -302,20 +423,29 @@ static int gsmtap_pdch(struct osmo_phsap_prim *l1sap, uint8_t *chan_type,
 static int gsmtap_ph_rach(struct osmo_phsap_prim *l1sap, uint8_t *chan_type,
 	uint8_t *tn, uint8_t *ss, uint32_t *fn, uint8_t **data, unsigned int *len)
 {
-	uint8_t chan_nr;
+	uint8_t chan_nr = l1sap->u.rach_ind.chan_nr;
 
 	*chan_type = GSMTAP_CHANNEL_RACH;
 	*fn = l1sap->u.rach_ind.fn;
-	*tn = L1SAP_CHAN2TS(l1sap->u.rach_ind.chan_nr);
-	chan_nr = l1sap->u.rach_ind.chan_nr;
+	*tn = L1SAP_CHAN2TS(chan_nr);
+
 	if (L1SAP_IS_CHAN_TCHH(chan_nr))
 		*ss = L1SAP_CHAN2SS_TCHH(chan_nr);
 	else if (L1SAP_IS_CHAN_SDCCH4(chan_nr))
 		*ss = L1SAP_CHAN2SS_SDCCH4(chan_nr);
 	else if (L1SAP_IS_CHAN_SDCCH8(chan_nr))
 		*ss = L1SAP_CHAN2SS_SDCCH8(chan_nr);
+	else if (L1SAP_IS_CHAN_PDCH(chan_nr)) {
+		if (L1SAP_IS_PTCCH(*fn)) {
+			/* TODO: calculate sub-slot from frame-number */
+			*chan_type = GSMTAP_CHANNEL_PTCCH;
+		} else {
+			*chan_type = GSMTAP_CHANNEL_PDTCH;
+		}
+	}
+
 	*data = (uint8_t *)&l1sap->u.rach_ind.ra;
-	*len = 1;
+	*len = (l1sap->u.rach_ind.is_11bit) ? 2 : 1;
 
 	return 0;
 }
@@ -328,6 +458,9 @@ static const uint8_t paging_fill[GSM_MACBLOCK_LEN] = {
 
 static bool is_fill_frame(uint8_t chan_type, const uint8_t *data, unsigned int len)
 {
+	if (len != GSM_MACBLOCK_LEN)
+		return false;
+
 	switch (chan_type) {
 	case GSMTAP_CHANNEL_AGCH:
 		if (!memcmp(data, fill_frame, GSM_MACBLOCK_LEN))
@@ -437,10 +570,7 @@ static unsigned int calc_exprd_rach_frames(struct gsm_bts *bts, uint32_t fn)
 			rach_frames_expired = 1;
 	}
 
-	/* Each Frame has room for 4 RACH slots, since RACH
-	 * slots are short enough to fit into a single radio
-	 * burst, so we need to multiply the final result by 4 */
-	return rach_frames_expired * 4;
+	return rach_frames_expired;
 }
 
 /* time information received from bts model */
@@ -449,6 +579,7 @@ static int l1sap_info_time_ind(struct gsm_bts *bts,
 			       struct info_time_ind_param *info_time_ind)
 {
 	int frames_expired;
+	int i;
 
 	DEBUGPFN(DL1P, info_time_ind->fn, "Rx MPH_INFO time ind\n");
 
@@ -469,8 +600,10 @@ static int l1sap_info_time_ind(struct gsm_bts *bts,
 
 	/* increment number of RACH slots that have passed by since the
 	 * last time indication */
-	bts->load.rach.total +=
-	    calc_exprd_rach_frames(bts, info_time_ind->fn) * frames_expired;
+	for (i = 0; i < frames_expired; i++) {
+		uint32_t fn = (info_time_ind->fn + GSM_HYPERFRAME - i) % GSM_HYPERFRAME;
+		bts->load.rach.total += calc_exprd_rach_frames(bts, fn);
+	}
 
 	return 0;
 }
@@ -481,8 +614,8 @@ static inline void set_ms_to_data(struct gsm_lchan *lchan, int16_t data, bool se
 		return;
 
 	if (data + 63 > 255) { /* According to 3GPP TS 48.058 §9.3.37 Timing Offset field cannot exceed 255 */
-		LOGP(DL1P, LOGL_ERROR, "Attempting to set invalid Timing Offset value %d (MS TO = %u)!\n",
-		     data, set_ms_to);
+		LOGPLCHAN(lchan, DL1P, LOGL_ERROR, "Attempting to set invalid Timing Offset value "
+			  "%d (MS TO = %u)!\n", data, set_ms_to);
 		return;
 	}
 
@@ -506,7 +639,7 @@ static int l1sap_info_meas_ind(struct gsm_bts_trx *trx,
 	lchan = get_active_lchan_by_chan_nr(trx, info_meas_ind->chan_nr);
 	if (!lchan) {
 		LOGPFN(DL1P, LOGL_ERROR, info_meas_ind->fn,
-			"No lchan for MPH INFO MEAS IND (chan_nr=%u)\n", info_meas_ind->chan_nr);
+			"No lchan for MPH INFO MEAS IND (chan_nr=%s)\n", rsl_chan_nr_str(info_meas_ind->chan_nr));
 		return 0;
 	}
 
@@ -529,16 +662,12 @@ static int l1sap_info_meas_ind(struct gsm_bts_trx *trx,
 	/* we assume that symbol period is 1 bit: */
 	set_ms_to_data(lchan, info_meas_ind->ta_offs_256bits / 256, true);
 
-	lchan_new_ul_meas(lchan, &ulm, info_meas_ind->fn);
-
-	/* Check measurement period end and prepare the UL measurment
-	 * report at Meas period End*/
-	lchan_meas_check_compute(lchan, info_meas_ind->fn);
+	lchan_meas_process_measurement(lchan, &ulm, info_meas_ind->fn);
 
 	return 0;
 }
 
-/* any L1 MPH_INFO indication prim recevied from bts model */
+/* any L1 MPH_INFO indication prim received from bts model */
 static int l1sap_mph_info_ind(struct gsm_bts_trx *trx,
 	 struct osmo_phsap_prim *l1sap, struct mph_info_param *info)
 {
@@ -572,12 +701,10 @@ static int l1sap_info_act_cnf(struct gsm_bts_trx *trx,
 	struct osmo_phsap_prim *l1sap,
 	struct info_act_cnf_param *info_act_cnf)
 {
-	struct gsm_lchan *lchan;
+	struct gsm_lchan *lchan = get_lchan_by_chan_nr(trx, info_act_cnf->chan_nr);
 
-	LOGP(DL1C, LOGL_INFO, "activate confirm chan_nr=0x%02x trx=%d\n",
-		info_act_cnf->chan_nr, trx->nr);
-
-	lchan = get_lchan_by_chan_nr(trx, info_act_cnf->chan_nr);
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "activate confirm chan_nr=%s trx=%d\n",
+		  rsl_chan_nr_str(info_act_cnf->chan_nr), trx->nr);
 
 	rsl_tx_chan_act_acknack(lchan, info_act_cnf->cause);
 
@@ -597,12 +724,10 @@ static int l1sap_info_rel_cnf(struct gsm_bts_trx *trx,
 	struct osmo_phsap_prim *l1sap,
 	struct info_act_cnf_param *info_act_cnf)
 {
-	struct gsm_lchan *lchan;
+	struct gsm_lchan *lchan = get_lchan_by_chan_nr(trx, info_act_cnf->chan_nr);
 
-	LOGP(DL1C, LOGL_INFO, "deactivate confirm chan_nr=0x%02x trx=%d\n",
-		info_act_cnf->chan_nr, trx->nr);
-
-	lchan = get_lchan_by_chan_nr(trx, info_act_cnf->chan_nr);
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "deactivate confirm chan_nr=%s trx=%d\n",
+		  rsl_chan_nr_str(info_act_cnf->chan_nr), trx->nr);
 
 	rsl_tx_rf_rel_ack(lchan);
 
@@ -616,7 +741,7 @@ static int l1sap_info_rel_cnf(struct gsm_bts_trx *trx,
 	return 0;
 }
 
-/* any L1 MPH_INFO confirm prim recevied from bts model */
+/* any L1 MPH_INFO confirm prim received from bts model */
 static int l1sap_mph_info_cnf(struct gsm_bts_trx *trx,
 	 struct osmo_phsap_prim *l1sap, struct mph_info_param *info)
 {
@@ -671,6 +796,42 @@ static int lchan_pdtch_ph_rts_ind_loop(struct gsm_lchan *lchan,
 	return 0;
 }
 
+/* Check if given CCCH frame number is for a PCH or for an AGCH (this function is
+ * only used internally, it is public to call it from unit-tests) */
+int is_ccch_for_agch(struct gsm_bts_trx *trx, uint32_t fn) {
+	/* Note: The number of available access grant channels is set by the
+	 * parameter BS_AG_BLKS_RES via system information type 3. This SI is
+	 * transferred to osmo-bts via RSL */
+        return l1sap_fn2ccch_block(fn) < num_agch(trx, "PH-RTS-IND");
+}
+
+/* return the measured average of frame numbers that the RTS clock is running in advance */
+int32_t bts_get_avg_fn_advance(struct gsm_bts *bts)
+{
+	if (bts->fn_stats.avg_count == 0)
+		return 0;
+	return bts->fn_stats.avg256 / bts->fn_stats.avg_count;
+}
+
+static void l1sap_update_fnstats(struct gsm_bts *bts, uint32_t rts_fn)
+{
+	int32_t delta = (rts_fn + GSM_HYPERFRAME - bts->gsm_time.fn) % GSM_HYPERFRAME;
+
+	if (delta < bts->fn_stats.min)
+		bts->fn_stats.min = delta;
+	if (delta > bts->fn_stats.max)
+		bts->fn_stats.max = delta;
+
+	if (bts->fn_stats.avg_count > bts->fn_stats.avg_window) {
+		/* reset and start old average and new sample */
+		bts->fn_stats.avg256 = (bts->fn_stats.avg256 / bts->fn_stats.avg_count) + delta;
+		bts->fn_stats.avg_count = 2;
+	} else {
+		bts->fn_stats.avg256 += delta;
+		bts->fn_stats.avg_count++;
+	}
+}
+
 /* PH-RTS-IND prim received from bts model */
 static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 	struct osmo_phsap_prim *l1sap, struct ph_data_param *rts_ind)
@@ -686,6 +847,7 @@ static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 	struct osmo_phsap_prim pp;
 	bool dtxd_facch = false;
 	int rc;
+	int is_ag_res;
 
 	chan_nr = rts_ind->chan_nr;
 	link_id = rts_ind->link_id;
@@ -694,7 +856,9 @@ static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 
 	gsm_fn2gsmtime(&g_time, fn);
 
-	DEBUGPGT(DL1P, &g_time, "Rx PH-RTS.ind chan_nr=0x%02x link_id=0x%02xd\n", chan_nr, link_id);
+	DEBUGPGT(DL1P, &g_time, "Rx PH-RTS.ind chan_nr=%s link_id=0x%02xd\n", rsl_chan_nr_str(chan_nr), link_id);
+
+	l1sap_update_fnstats(trx->bts, fn);
 
 	/* reuse PH-RTS.ind for PH-DATA.req */
 	if (!msg) {
@@ -715,10 +879,10 @@ static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 		} else {
 			/* forward RTS.ind to PCU */
 			if (L1SAP_IS_PTCCH(rts_ind->fn)) {
-				pcu_tx_rts_req(&trx->ts[tn], 1, fn, 1 /* ARFCN */,
+				pcu_tx_rts_req(&trx->ts[tn], 1, fn, trx->arfcn,
 						L1SAP_FN2PTCCHBLOCK(fn));
 			} else {
-				pcu_tx_rts_req(&trx->ts[tn], 0, fn, 0 /* ARFCN */,
+				pcu_tx_rts_req(&trx->ts[tn], 0, fn, trx->arfcn,
 						L1SAP_FN2MACBLOCK(fn));
 			}
 			/* return early, PCU takes care of rest */
@@ -732,10 +896,14 @@ static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 			memcpy(p, si, GSM_MACBLOCK_LEN);
 		else
 			memcpy(p, fill_frame, GSM_MACBLOCK_LEN);
+	} else if (L1SAP_IS_CHAN_CBCH(chan_nr)) {
+		p = msgb_put(msg, GSM_MACBLOCK_LEN);
+		bts_cbch_get(trx->bts, p, &g_time);
 	} else if (!(chan_nr & 0x80)) { /* only TCH/F, TCH/H, SDCCH/4 and SDCCH/8 have C5 bit cleared */
 		lchan = get_active_lchan_by_chan_nr(trx, chan_nr);
 		if (!lchan) {
-			LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for PH-RTS.ind (chan_nr=%u)\n", chan_nr);
+			LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for PH-RTS.ind (chan_nr=%s)\n",
+			       rsl_chan_nr_str(chan_nr));
 			return 0;
 		}
 		if (L1SAP_IS_LINK_SACCH(link_id)) {
@@ -759,9 +927,16 @@ static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 					memcpy(p + 2, si, GSM_MACBLOCK_LEN - 2);
 				} else
 					memcpy(p + 2, fill_frame, GSM_MACBLOCK_LEN - 2);
-			} else if ((!L1SAP_IS_CHAN_TCHF(chan_nr) && !L1SAP_IS_CHAN_TCHH(chan_nr))
-				|| lchan->rsl_cmode == RSL_CMOD_SPD_SIGN) {
-				/* send fill frame only, if not TCH/x != Signalling, otherwise send empty frame */
+			} else if (L1SAP_IS_CHAN_SDCCH4(chan_nr) || L1SAP_IS_CHAN_SDCCH8(chan_nr) ||
+				   (lchan->rsl_cmode == RSL_CMOD_SPD_SIGN && !lchan->ts->trx->bts->dtxd)) {
+				/*
+				 * SDCCH or TCH in signalling mode without DTX.
+				 *
+				 * Send fill frame according to GSM 05.08, section 8.3: "On the SDCCH and on the
+				 * half rate speech traffic channel in signalling only mode DTX is not allowed.
+				 * In these cases and during signalling on the TCH when DTX is not used, the same
+				 * L2 fill frame shall be transmitted in case there is nothing else to transmit."
+				 */
 				p = msgb_put(msg, GSM_MACBLOCK_LEN);
 				memcpy(p, fill_frame, GSM_MACBLOCK_LEN);
 			} /* else the message remains empty, so TCH frames are sent */
@@ -781,14 +956,13 @@ static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 		}
 	} else if (L1SAP_IS_CHAN_AGCH_PCH(chan_nr)) {
 		p = msgb_put(msg, GSM_MACBLOCK_LEN);
-		rc = bts_ccch_copy_msg(trx->bts, p, &g_time,
-				       (L1SAP_FN2CCCHBLOCK(fn) <
-					num_agch(trx, "PH-RTS-IND")));
+		is_ag_res = is_ccch_for_agch(trx, fn);
+		rc = bts_ccch_copy_msg(trx->bts, p, &g_time, is_ag_res);
 		if (rc <= 0)
 			memcpy(p, fill_frame, GSM_MACBLOCK_LEN);
 	}
 
-	DEBUGPGT(DL1P, &g_time, "Tx PH-DATA.req chan_nr=0x%02x link_id=0x%02x\n", chan_nr, link_id);
+	DEBUGPGT(DL1P, &g_time, "Tx PH-DATA.req chan_nr=%s link_id=0x%02x\n", rsl_chan_nr_str(chan_nr), link_id);
 
 	l1sap_down(trx, l1sap);
 
@@ -829,15 +1003,15 @@ static bool rtppayload_is_valid(struct gsm_lchan *lchan, struct msgb *resp_msg)
 	 * don't support it. */
 	if(lchan->tch_mode == GSM48_CMODE_SPEECH_AMR &&
 		!rtppayload_is_octet_aligned(resp_msg->data, resp_msg->len)) {
-		LOGP(DL1P, LOGL_NOTICE,
-			"%s RTP->L1: Dropping unexpected AMR encoding (bw-efficient?) %s\n",
-			gsm_lchan_name(lchan), osmo_hexdump(resp_msg->data, resp_msg->len));
+		LOGPLCHAN(lchan, DL1P, LOGL_NOTICE,
+			  "RTP->L1: Dropping unexpected AMR encoding (bw-efficient?) %s\n",
+			  osmo_hexdump(resp_msg->data, resp_msg->len));
 		return false;
 	}
 	return true;
 }
 
-/* TCH-RTS-IND prim recevied from bts model */
+/* TCH-RTS-IND prim received from bts model */
 static int l1sap_tch_rts_ind(struct gsm_bts_trx *trx,
 	struct osmo_phsap_prim *l1sap, struct ph_tch_param *rts_ind)
 {
@@ -854,11 +1028,11 @@ static int l1sap_tch_rts_ind(struct gsm_bts_trx *trx,
 
 	gsm_fn2gsmtime(&g_time, fn);
 
-	DEBUGPGT(DL1P, &g_time, "Rx TCH-RTS.ind chan_nr=0x%02x\n", chan_nr);
+	DEBUGPGT(DL1P, &g_time, "Rx TCH-RTS.ind chan_nr=%s\n", rsl_chan_nr_str(chan_nr));
 
 	lchan = get_active_lchan_by_chan_nr(trx, chan_nr);
 	if (!lchan) {
-		LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for PH-RTS.ind (chan_nr=%u)\n", chan_nr);
+		LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for PH-RTS.ind (chan_nr=%s)\n", rsl_chan_nr_str(chan_nr));
 		return 0;
 	}
 
@@ -908,7 +1082,7 @@ static int l1sap_tch_rts_ind(struct gsm_bts_trx *trx,
 	resp_l1sap->u.tch.fn = fn;
 	resp_l1sap->u.tch.marker = marker;
 
-	DEBUGPGT(DL1P, &g_time, "Tx TCH.req chan_nr=0x%02x\n", chan_nr);
+	DEBUGPGT(DL1P, &g_time, "Tx TCH.req chan_nr=%s\n", rsl_chan_nr_str(chan_nr));
 
 	l1sap_down(trx, resp_l1sap);
 
@@ -938,8 +1112,12 @@ static void radio_link_timeout(struct gsm_lchan *lchan, int bad_frame)
 		lchan->s--;
 		DEBUGP(DMEAS, "%s counting down radio link counter S=%d\n",
 			gsm_lchan_name(lchan), lchan->s);
-		if (lchan->s == 0)
+		if (lchan->s == 0) {
+			LOGPLCHAN(lchan, DMEAS, LOGL_NOTICE,
+				  "radio link counter timeout S=%d, dropping conn\n",
+				  lchan->s);
 			rsl_tx_conn_fail(lchan, RSL_ERR_RADIO_LINK_FAIL);
+		}
 		return;
 	}
 
@@ -956,7 +1134,7 @@ static void radio_link_timeout(struct gsm_lchan *lchan, int bad_frame)
 static inline int check_for_first_ciphrd(struct gsm_lchan *lchan,
 					  uint8_t *data, int len)
 {
-	uint8_t n_s;
+	uint8_t n_r;
 
 	/* if this is the first valid message after enabling Rx
 	 * decryption, we have to enable Tx encryption */
@@ -972,8 +1150,8 @@ static inline int check_for_first_ciphrd(struct gsm_lchan *lchan,
 	if ((data[1] & 0x01) != 0)
 		return 0;
 
-	n_s = data[1] >> 5;
-	if (lchan->ciph_ns != n_s)
+	n_r = data[1] >> 5;
+	if (lchan->ciph_ns != n_r)
 		return 0;
 
 	return 1;
@@ -1010,14 +1188,23 @@ static int l1sap_ph_data_ind(struct gsm_bts_trx *trx,
 
 	gsm_fn2gsmtime(&g_time, fn);
 
-	DEBUGPGT(DL1P, &g_time, "Rx PH-DATA.ind chan_nr=0x%02x link_id=0x%02x len=%d\n",
-		 chan_nr, link_id, len);
+	DEBUGPGT(DL1P, &g_time, "Rx PH-DATA.ind chan_nr=%s link_id=0x%02x len=%d\n",
+		 rsl_chan_nr_str(chan_nr), link_id, len);
+
+	/* Actually, there can be no DATA.ind on PTCCH/U (rather RACH.ind instead),
+	 * but some BTS models with buggy implementation may still be sending them
+	 * to us. Let's keep this for backwards compatibility. */
+	if (L1SAP_IS_CHAN_PDCH(chan_nr) && L1SAP_IS_PTCCH(fn)) {
+		LOGPGT(DL1P, LOGL_NOTICE, &g_time, "There can be no DATA.ind on PTCCH/U. "
+		       "This is probably a bug of the BTS model you're using, please fix!\n");
+		return -EINVAL;
+	}
 
 	if (ts_is_pdch(&trx->ts[tn])) {
 		lchan = get_lchan_by_chan_nr(trx, chan_nr);
 		if (!lchan)
-			LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for chan_nr=0x%02x\n", chan_nr);
-		if (lchan && lchan->loopback && !L1SAP_IS_PTCCH(fn)) {
+			LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for chan_nr=%s\n", rsl_chan_nr_str(chan_nr));
+		if (lchan && lchan->loopback) {
 			/* we are in loopback mode (for BER testing)
 			 * mode and need to enqeue the frame to be
 			 * returned in downlink */
@@ -1032,34 +1219,38 @@ static int l1sap_ph_data_ind(struct gsm_bts_trx *trx,
 		/* don't send bad frames to PCU */
 		if (len == 0)
 			return -EINVAL;
-		if (L1SAP_IS_PTCCH(fn)) {
-			pcu_tx_data_ind(&trx->ts[tn], PCU_IF_SAPI_PTCCH, fn,
-					0 /* ARFCN */, L1SAP_FN2PTCCHBLOCK(fn),
-					data, len, rssi, data_ind->ber10k,
-					data_ind->ta_offs_256bits/64,
-					data_ind->lqual_cb);
-		} else {
-			/* drop incomplete UL block */
-			if (pr_info != PRES_INFO_BOTH)
-				return 0;
-			/* PDTCH / PACCH frame handling */
-			pcu_tx_data_ind(&trx->ts[tn], PCU_IF_SAPI_PDTCH, fn, 0 /* ARFCN */,
-					L1SAP_FN2MACBLOCK(fn), data, len, rssi, data_ind->ber10k,
-					data_ind->ta_offs_256bits/64, data_ind->lqual_cb);
-		}
+		/* drop incomplete UL block */
+		if (pr_info != PRES_INFO_BOTH)
+			return 0;
+
+		/* PDTCH / PACCH frame handling */
+		pcu_tx_data_ind(&trx->ts[tn], PCU_IF_SAPI_PDTCH, fn, trx->arfcn,
+				L1SAP_FN2MACBLOCK(fn), data, len, rssi, data_ind->ber10k,
+				data_ind->ta_offs_256bits/64, data_ind->lqual_cb);
 		return 0;
 	}
 
 	lchan = get_active_lchan_by_chan_nr(trx, chan_nr);
 	if (!lchan) {
-		LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for chan_nr=%d\n", chan_nr);
+		LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for chan_nr=%s\n", rsl_chan_nr_str(chan_nr));
 		return 0;
 	}
 
 	/* bad frame */
 	if (len == 0) {
-		if (L1SAP_IS_LINK_SACCH(link_id))
+		if (L1SAP_IS_LINK_SACCH(link_id)) {
+			/* In case we loose a SACCH block, we must take care
+			 * that the related measurement report is sent via RSL.
+			 * This is a fallback method. The report will also
+			 * lack the measurement report from the MS side. See
+			 * also rsl.c:lapdm_rll_tx_cb() */
+			LOGPGT(DL1P, LOGL_INFO, &g_time, "Lost SACCH block, faking meas reports and ms pwr\n");
+			le = &lchan->lapdm_ch.lapdm_acch;
+			rsl_tx_meas_res(lchan, NULL, 0, le);
+
 			radio_link_timeout(lchan, 1);
+			lchan_ms_pwr_ctrl(lchan, lchan->ms_power_ctrl.current, data_ind->rssi);
+		}
 		return -EINVAL;
 	}
 
@@ -1077,7 +1268,7 @@ static int l1sap_ph_data_ind(struct gsm_bts_trx *trx,
 		}
 		/* Some brilliant engineer decided that the ordering of
 		 * fields on the Um interface is different from the
-		 * order of fields in RLS. See TS 04.04 (Chapter 7.2)
+		 * order of fields in RSL. See TS 04.04 (Chapter 7.2)
 		 * vs. TS 08.58 (Chapter 9.3.10). */
 		lchan->meas.l1_info[0] = data[0] << 3;
 		lchan->meas.l1_info[0] |= ((data[0] >> 5) & 1) << 2;
@@ -1116,11 +1307,11 @@ static int l1sap_tch_ind(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap,
 
 	gsm_fn2gsmtime(&g_time, fn);
 
-	LOGPGT(DL1P, LOGL_INFO, &g_time, "Rx TCH.ind chan_nr=0x%02x\n", chan_nr);
+	LOGPGT(DL1P, LOGL_INFO, &g_time, "Rx TCH.ind chan_nr=%s\n", rsl_chan_nr_str(chan_nr));
 
 	lchan = get_active_lchan_by_chan_nr(trx, chan_nr);
 	if (!lchan) {
-		LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for TCH.ind (chan_nr=%u)\n", chan_nr);
+		LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for TCH.ind (chan_nr=%s)\n", rsl_chan_nr_str(chan_nr));
 		return 0;
 	}
 
@@ -1130,7 +1321,7 @@ static int l1sap_tch_ind(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap,
 	 * the content is not available due to decoding issues. Content not
 	 * available is expected as empty payload. We also check if quality is
 	 * good enough. */
-	if (msg->len && tch_ind->lqual_cb / 10 >= bts->min_qual_norm) {
+	if (msg->len && tch_ind->lqual_cb >= bts->min_qual_norm) {
 		/* hand msg to RTP code for transmission */
 		if (lchan->abis_ip.rtp_socket)
 			osmo_rtp_send_frame_ext(lchan->abis_ip.rtp_socket,
@@ -1160,14 +1351,15 @@ static int l1sap_tch_ind(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap,
 
 #define RACH_MIN_TOA256 -2 * 256
 
-static bool rach_pass_filter(struct ph_rach_ind_param *rach_ind, struct gsm_bts *bts)
+static bool rach_pass_filter(struct ph_rach_ind_param *rach_ind, struct gsm_bts *bts,
+			     const char *chan_name)
 {
 	int16_t toa256 = rach_ind->acc_delay_256bits;
 
 	/* Check for RACH exceeding BER threshold (ghost RACH) */
 	if (rach_ind->ber10k > bts->max_ber10k_rach) {
-		LOGPFN(DL1C, LOGL_INFO, rach_ind->fn, "Ignoring RACH request: "
-			"BER10k(%u) > BER10k_MAX(%u)\n",
+		LOGPFN(DL1C, LOGL_INFO, rach_ind->fn, "Ignoring an Access Burst on %s: "
+			"BER10k(%u) > BER10k_MAX(%u)\n", chan_name,
 			rach_ind->ber10k, bts->max_ber10k_rach);
 		return false;
 	}
@@ -1178,21 +1370,28 @@ static bool rach_pass_filter(struct ph_rach_ind_param *rach_ind, struct gsm_bts
 	 * according to maximal allowed Timing Advance value.
 	 */
 	if (toa256 < RACH_MIN_TOA256 || toa256 > bts->max_ta * 256) {
-		LOGPFN(DL1C, LOGL_INFO, rach_ind->fn, "Ignoring RACH request: "
-			"ToA(%d) exceeds the allowed range (%d..%d)\n",
+		LOGPFN(DL1C, LOGL_INFO, rach_ind->fn, "Ignoring an Access Burst on %s: "
+			"ToA(%d) exceeds the allowed range (%d..%d)\n", chan_name,
 			toa256, RACH_MIN_TOA256, bts->max_ta * 256);
 		return false;
 	}
 
+	/* Link quality defined by C/I (Carrier-to-Interference ratio) */
+	if (rach_ind->lqual_cb < bts->min_qual_rach) {
+		LOGPFN(DL1C, LOGL_INFO, rach_ind->fn, "Ignoring an Access Burst on %s: "
+			"link quality (%d) below the minimum (%d)\n", chan_name,
+			rach_ind->lqual_cb, bts->min_qual_rach);
+		return false;
+	}
+
 	return true;
 }
 
 /* Special case where handover RACH is detected */
-static int l1sap_handover_rach(struct gsm_bts_trx *trx,
-	struct osmo_phsap_prim *l1sap, struct ph_rach_ind_param *rach_ind)
+static int l1sap_handover_rach(struct gsm_bts_trx *trx, struct ph_rach_ind_param *rach_ind)
 {
 	/* Filter out noise / interference / ghosts */
-	if (!rach_pass_filter(rach_ind, trx->bts)) {
+	if (!rach_pass_filter(rach_ind, trx->bts, "handover")) {
 		rate_ctr_inc2(trx->bts->ctrs, BTS_CTR_RACH_DROP);
 		return 0;
 	}
@@ -1204,6 +1403,33 @@ static int l1sap_handover_rach(struct gsm_bts_trx *trx,
 	return 0;
 }
 
+/* Special case for Access Bursts on PDTCH/U or PTCCH/U */
+static int l1sap_pdch_rach(struct gsm_bts_trx *trx, struct ph_rach_ind_param *rach_ind)
+{
+	/* Filter out noise / interference / ghosts */
+	if (!rach_pass_filter(rach_ind, trx->bts, "PDCH"))
+		return -EAGAIN;
+
+	/* PTCCH/U (Packet Timing Advance Control Channel) */
+	if (L1SAP_IS_PTCCH(rach_ind->fn)) {
+		LOGPFN(DL1P, LOGL_DEBUG, rach_ind->fn,
+		       /* TODO: calculate and print Timing Advance Index */
+		       "Access Burst for continuous Timing Advance control (toa256=%d)\n",
+		       rach_ind->acc_delay_256bits);
+
+		/* QTA: Timing Advance in units of 1/4 of a symbol */
+		pcu_tx_rach_ind(trx->bts->nr, trx->nr, rach_ind->chan_nr & 0x07,
+				rach_ind->acc_delay_256bits >> 6,
+				rach_ind->ra, rach_ind->fn, rach_ind->is_11bit,
+				rach_ind->burst_type, PCU_IF_SAPI_PTCCH);
+		return 0;
+	} else { /* The MS may acknowledge DL data by 4 consequent Access Bursts */
+		LOGPFN(DL1P, LOGL_NOTICE, rach_ind->fn,
+		       "Access Bursts on PDTCH/U are not (yet) supported\n");
+		return -ENOTSUP;
+	}
+}
+
 /* RACH received from bts model */
 static int l1sap_ph_rach_ind(struct gsm_bts_trx *trx,
 	 struct osmo_phsap_prim *l1sap, struct ph_rach_ind_param *rach_ind)
@@ -1211,12 +1437,19 @@ static int l1sap_ph_rach_ind(struct gsm_bts_trx *trx,
 	struct gsm_bts *bts = trx->bts;
 	struct lapdm_channel *lc;
 
-	DEBUGPFN(DL1P, rach_ind->fn, "Rx PH-RA.ind");
+	DEBUGPFN(DL1P, rach_ind->fn, "Rx PH-RA.ind\n");
 
-	/* check for handover access burst on dedicated channels */
-	if (!L1SAP_IS_CHAN_RACH(rach_ind->chan_nr)) {
+	/* Check the origin of an Access Burst */
+	switch (rach_ind->chan_nr & 0xf8) {
+	case RSL_CHAN_RACH:
+		/* CS or PS RACH, to be handled in this function */
+		break;
+	case RSL_CHAN_OSMO_PDCH:
+		/* TODO: do we need to count Access Bursts on PDCH? */
+		return l1sap_pdch_rach(trx, rach_ind);
+	default:
 		rate_ctr_inc2(trx->bts->ctrs, BTS_CTR_RACH_HO);
-		return l1sap_handover_rach(trx, l1sap, rach_ind);
+		return l1sap_handover_rach(trx, rach_ind);
 	}
 
 	rate_ctr_inc2(trx->bts->ctrs, BTS_CTR_RACH_RCVD);
@@ -1226,7 +1459,7 @@ static int l1sap_ph_rach_ind(struct gsm_bts_trx *trx,
 		bts->load.rach.busy++;
 
 	/* Filter out noise / interference / ghosts */
-	if (!rach_pass_filter(rach_ind, bts)) {
+	if (!rach_pass_filter(rach_ind, bts, "CCCH")) {
 		rate_ctr_inc2(trx->bts->ctrs, BTS_CTR_RACH_DROP);
 		return 0;
 	}
@@ -1248,9 +1481,11 @@ static int l1sap_ph_rach_ind(struct gsm_bts_trx *trx,
 		LOGPFN(DL1P, LOGL_INFO, rach_ind->fn, "RACH for packet access (toa=%d, ra=%d)\n",
 			rach_ind->acc_delay, rach_ind->ra);
 
-		pcu_tx_rach_ind(bts, rach_ind->acc_delay << 2,
-			rach_ind->ra, rach_ind->fn,
-			rach_ind->is_11bit, rach_ind->burst_type);
+		/* QTA: Timing Advance in units of 1/4 of a symbol */
+		pcu_tx_rach_ind(bts->nr, trx->nr, rach_ind->chan_nr & 0x07,
+			rach_ind->acc_delay_256bits >> 6,
+			rach_ind->ra, rach_ind->fn, rach_ind->is_11bit,
+			rach_ind->burst_type, PCU_IF_SAPI_RACH);
 		return 0;
 	}
 
@@ -1299,8 +1534,10 @@ int l1sap_up(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 	default:
 		LOGP(DL1P, LOGL_NOTICE, "unknown prim %d op %d\n",
 			l1sap->oph.primitive, l1sap->oph.operation);
-		oml_fail_rep(OSMO_EVT_MAJ_UKWN_MSG, "unknown prim %d op %d",
-			     l1sap->oph.primitive, l1sap->oph.operation);
+		oml_tx_failure_event_rep(&trx->mo, NM_SEVER_MAJOR, OSMO_EVT_MAJ_UKWN_MSG,
+					 "unknown prim %d op %d",
+					 l1sap->oph.primitive,
+					 l1sap->oph.operation);
 		break;
 	}
 
@@ -1314,6 +1551,9 @@ int l1sap_up(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 /* any L1 prim sent to bts model */
 static int l1sap_down(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 {
+	l1sap_log_ctx_sapi = get_common_sapi_by_trx_prim(trx, l1sap);
+	log_set_context(LOG_CTX_L1_SAPI, &l1sap_log_ctx_sapi);
+
 	if (OSMO_PRIM_HDR(&l1sap->oph) ==
 				 OSMO_PRIM(PRIM_PH_DATA, PRIM_OP_REQUEST))
 		to_gsmtap(trx, l1sap);
@@ -1323,7 +1563,7 @@ static int l1sap_down(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 
 /* pcu (socket interface) sends us a data request primitive */
 int l1sap_pdch_req(struct gsm_bts_trx_ts *ts, int is_ptcch, uint32_t fn,
-	uint16_t arfcn, uint8_t block_nr, uint8_t *data, uint8_t len)
+	uint16_t arfcn, uint8_t block_nr, const uint8_t *data, uint8_t len)
 {
 	struct msgb *msg;
 	struct osmo_phsap_prim *l1sap;
@@ -1402,8 +1642,8 @@ int l1sap_chan_act(struct gsm_bts_trx *trx, uint8_t chan_nr, struct tlv_parsed *
 	struct gsm48_chan_desc *cd;
 	int rc;
 
-	LOGP(DL1C, LOGL_INFO, "activating channel chan_nr=0x%02x trx=%d\n",
-		chan_nr, trx->nr);
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "activating channel chan_nr=%s trx=%d\n",
+		  rsl_chan_nr_str(chan_nr), trx->nr);
 
 	/* osmo-pcu calls this without a valid 'tp' parameter, so we
 	 * need to make sure ew don't crash here */
@@ -1416,8 +1656,8 @@ int l1sap_chan_act(struct gsm_bts_trx *trx, uint8_t chan_nr, struct tlv_parsed *
 		 * one one TRX, so we need to make sure not to activate
 		 * channels with a different TSC!! */
 		if (cd->h0.tsc != (lchan->ts->trx->bts->bsic & 7)) {
-			LOGP(DL1C, LOGL_ERROR, "lchan TSC %u != BSIC-TSC %u\n",
-				cd->h0.tsc, lchan->ts->trx->bts->bsic & 7);
+			LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "lchan TSC %u != BSIC-TSC %u\n",
+				  cd->h0.tsc, lchan->ts->trx->bts->bsic & 7);
 			return -RSL_ERR_SERV_OPT_UNIMPL;
 		}
 	}
@@ -1450,8 +1690,8 @@ int l1sap_chan_act(struct gsm_bts_trx *trx, uint8_t chan_nr, struct tlv_parsed *
 int l1sap_chan_rel(struct gsm_bts_trx *trx, uint8_t chan_nr)
 {
 	struct gsm_lchan *lchan = get_lchan_by_chan_nr(trx, chan_nr);
-	LOGP(DL1C, LOGL_INFO, "deactivating channel chan_nr=0x%02x trx=%d\n",
-		chan_nr, trx->nr);
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "deactivating channel chan_nr=%s trx=%d\n",
+		  rsl_chan_nr_str(chan_nr), trx->nr);
 
 	if (lchan->tch.dtx.dl_amr_fsm) {
 		osmo_fsm_inst_free(lchan->tch.dtx.dl_amr_fsm);
@@ -1466,8 +1706,8 @@ int l1sap_chan_deact_sacch(struct gsm_bts_trx *trx, uint8_t chan_nr)
 {
 	struct gsm_lchan *lchan = get_lchan_by_chan_nr(trx, chan_nr);
 
-	LOGP(DL1C, LOGL_INFO, "deactivating sacch chan_nr=0x%02x trx=%d\n",
-		chan_nr, trx->nr);
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "deactivating sacch chan_nr=%s trx=%d\n",
+		  rsl_chan_nr_str(chan_nr), trx->nr);
 
 	lchan->sacch_deact = 1;
 
@@ -1477,8 +1717,8 @@ int l1sap_chan_deact_sacch(struct gsm_bts_trx *trx, uint8_t chan_nr)
 
 int l1sap_chan_modify(struct gsm_bts_trx *trx, uint8_t chan_nr)
 {
-	LOGP(DL1C, LOGL_INFO, "modifying channel chan_nr=0x%02x trx=%d\n",
-		chan_nr, trx->nr);
+	LOGP(DL1C, LOGL_INFO, "modifying channel chan_nr=%s trx=%d\n",
+		rsl_chan_nr_str(chan_nr), trx->nr);
 
 	return l1sap_chan_act_dact_modify(trx, chan_nr, PRIM_INFO_MODIFY, 0);
 }
