@@ -33,6 +33,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
+#include <osmocom/core/stats.h>
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/application.h>
 #include <osmocom/vty/telnet_interface.h>
@@ -59,7 +60,6 @@ int quit = 0;
 static const char *config_file = "osmo-bts.cfg";
 static int daemonize = 0;
 static int rt_prio = -1;
-static int trx_num = 1;
 static char *gsmtap_ip = 0;
 extern int g_vty_port_num;
 
@@ -76,8 +76,6 @@ static void print_help()
 		"  -e 	--log-level	Set a global log-level\n"
 		"  -r	--realtime PRIO	Use SCHED_RR with the specified priority\n"
 		"  -i	--gsmtap-ip	The destination IP used for GSMTAP.\n"
-		"  -t	--trx-num	Set number of TRX (default=%d)\n",
-		trx_num
 		);
 	bts_model_print_help();
 }
@@ -152,9 +150,8 @@ static void handle_options(int argc, char **argv)
 			gsmtap_ip = optarg;
 			break;
 		case 't':
-			trx_num = atoi(optarg);
-			if (trx_num < 1)
-				trx_num = 1;
+			fprintf(stderr, "Parameter -t is deprecated and does nothing, "
+					"TRX num is calculated from VTY\n");
 			break;
 		case '?':
 		case 1:
@@ -186,8 +183,9 @@ static void signal_handler(int signal)
 	case SIGINT:
 	case SIGTERM:
 		if (!quit) {
-			oml_fail_rep(OSMO_EVT_CRIT_PROC_STOP,
-				     "BTS: SIGINT received -> shutdown");
+			oml_tx_failure_event_rep(&bts->mo,
+						 NM_SEVER_CRITICAL, OSMO_EVT_CRIT_PROC_STOP,
+						 "BTS: SIGINT received -> shutdown");
 			bts_shutdown(bts, "SIGINT");
 		}
 		quit++;
@@ -195,9 +193,6 @@ static void signal_handler(int signal)
 	case SIGABRT:
 	case SIGUSR1:
 	case SIGUSR2:
-		oml_fail_rep(OSMO_EVT_CRIT_PROC_STOP,
-			     "BTS: signal %d (%s) received", signal,
-			     strsignal(signal));
 		talloc_report_full(tall_bts_ctx, stderr);
 		break;
 	default:
@@ -228,7 +223,7 @@ int bts_main(int argc, char **argv)
 {
 	struct gsm_bts_trx *trx;
 	struct e1inp_line *line;
-	int rc, i;
+	int rc;
 
 	printf("((*))\n  |\n / \\ OsmoBTS\n");
 
@@ -240,6 +235,7 @@ int bts_main(int argc, char **argv)
 	bts_vty_info.tall_ctx = tall_bts_ctx;
 
 	osmo_init_logging2(tall_bts_ctx, &bts_log_info);
+	osmo_stats_init(tall_bts_ctx);
 	vty_init(&bts_vty_info);
 	ctrl_vty_init(tall_bts_ctx);
 	rate_ctr_init(tall_bts_ctx);
@@ -251,15 +247,9 @@ int bts_main(int argc, char **argv)
 		fprintf(stderr, "Failed to create BTS structure\n");
 		exit(1);
 	}
-	for (i = 1; i < trx_num; i++) {
-		trx = gsm_bts_trx_alloc(bts);
-		if (!trx) {
-			fprintf(stderr, "Failed to create TRX structure\n");
-			exit(1);
-		}
-	}
+
 	e1inp_vty_init();
-	bts_vty_init(bts, &bts_log_info);
+	bts_vty_init(bts);
 
 	/* enable realtime priority for us */
 	if (rt_prio != -1) {
