@@ -1,6 +1,6 @@
 /* pcu_sock.c: Connect from PCU via unix domain socket */
 
-/* (C) 2008-2010 by Harald Welte <laforge@gnumonks.org>
+/* (C) 2008-2019 by Harald Welte <laforge@gnumonks.org>
  * (C) 2009-2012 by Andreas Eversberg <jolly@eversberg.eu>
  * (C) 2012 by Holger Hans Peter Freyther
  * All Rights Reserved
@@ -43,6 +43,7 @@
 #include <osmo-bts/rsl.h>
 #include <osmo-bts/signal.h>
 #include <osmo-bts/l1sap.h>
+#include <osmo-bts/oml.h>
 
 uint32_t trx_get_hlayer1(struct gsm_bts_trx *trx);
 
@@ -258,7 +259,7 @@ static int pcu_if_signal_cb(unsigned int subsys, unsigned int signal,
 						bts->si_buf[SYSINFO_TYPE_3];
 		osmo_plmn_from_bcd(si3->lai.digits, &net->plmn);
 		bts->location_area_code = ntohs(si3->lai.lac);
-		bts->cell_identity = si3->cell_identity;
+		bts->cell_identity = ntohs(si3->cell_identity);
 		avail_lai = 1;
 		break;
 	case S_NEW_NSE_ATTR:
@@ -289,6 +290,27 @@ static int pcu_if_signal_cb(unsigned int subsys, unsigned int signal,
 	return 0;
 }
 
+int pcu_tx_app_info_req(struct gsm_bts *bts, uint8_t app_type, uint8_t len, const uint8_t *app_data)
+{
+	struct gsm_pcu_if_app_info_req *ai_req;
+	struct gsm_pcu_if *pcu_prim;
+	struct msgb *msg;
+
+	if (app_type & 0xF0 || len > sizeof(ai_req->data))
+		return -EINVAL;
+
+	msg = pcu_msgb_alloc(PCU_IF_MSG_APP_INFO_REQ, bts->nr);
+	if (!msg)
+		return -ENOMEM;
+	pcu_prim = (struct gsm_pcu_if *) msg->data;
+	ai_req = &pcu_prim->u.app_info_req;
+
+	ai_req->application_type = app_type;
+	ai_req->len = len;
+	memcpy(ai_req->data, app_data, ai_req->len);
+
+	return pcu_sock_send(&bts_gsmnet, msg);
+}
 
 int pcu_tx_rts_req(struct gsm_bts_trx_ts *ts, uint8_t is_ptcch, uint32_t fn,
 	uint16_t arfcn, uint8_t block_nr)
@@ -329,8 +351,8 @@ int pcu_tx_data_ind(struct gsm_bts_trx_ts *ts, uint8_t sapi, uint32_t fn,
 	LOGP(DPCU, LOGL_DEBUG, "Sending data indication: sapi=%s arfcn=%d block=%d data=%s\n",
 	     sapi_string[sapi], arfcn, block_nr, osmo_hexdump(data, len));
 
-	if (lqual / 10 < bts->min_qual_norm) {
-		LOGP(DPCU, LOGL_DEBUG, "Link quality %"PRId16" is below threshold %f, dropping packet\n",
+	if (lqual < bts->min_qual_norm) {
+		LOGP(DPCU, LOGL_DEBUG, "Link quality %"PRId16" is below threshold %d, dropping packet\n",
 			lqual, bts->min_qual_norm);
 		return 0;
 	}
@@ -357,8 +379,9 @@ int pcu_tx_data_ind(struct gsm_bts_trx_ts *ts, uint8_t sapi, uint32_t fn,
 	return pcu_sock_send(&bts_gsmnet, msg);
 }
 
-int pcu_tx_rach_ind(struct gsm_bts *bts, int16_t qta, uint16_t ra, uint32_t fn,
-	uint8_t is_11bit, enum ph_burst_type burst_type)
+int pcu_tx_rach_ind(uint8_t bts_nr, uint8_t trx_nr, uint8_t ts_nr,
+		    int16_t qta, uint16_t ra, uint32_t fn, uint8_t is_11bit,
+		    enum ph_burst_type burst_type, uint8_t sapi)
 {
 	struct msgb *msg;
 	struct gsm_pcu_if *pcu_prim;
@@ -367,18 +390,20 @@ int pcu_tx_rach_ind(struct gsm_bts *bts, int16_t qta, uint16_t ra, uint32_t fn,
 	LOGP(DPCU, LOGL_INFO, "Sending RACH indication: qta=%d, ra=%d, "
 		"fn=%d\n", qta, ra, fn);
 
-	msg = pcu_msgb_alloc(PCU_IF_MSG_RACH_IND, bts->nr);
+	msg = pcu_msgb_alloc(PCU_IF_MSG_RACH_IND, bts_nr);
 	if (!msg)
 		return -ENOMEM;
 	pcu_prim = (struct gsm_pcu_if *) msg->data;
 	rach_ind = &pcu_prim->u.rach_ind;
 
-	rach_ind->sapi = PCU_IF_SAPI_RACH;
+	rach_ind->sapi = sapi;
 	rach_ind->ra = ra;
 	rach_ind->qta = qta;
 	rach_ind->fn = fn;
 	rach_ind->is_11bit = is_11bit;
 	rach_ind->burst_type = burst_type;
+	rach_ind->trx_nr = trx_nr;
+	rach_ind->ts_nr = ts_nr;
 
 	return pcu_sock_send(&bts_gsmnet, msg);
 }
@@ -465,8 +490,25 @@ int pcu_tx_pch_data_cnf(uint32_t fn, uint8_t *data, uint8_t len)
 	return pcu_sock_send(&bts_gsmnet, msg);
 }
 
+/* forward data from a RR GPRS SUSPEND REQ towards PCU */
+int pcu_tx_susp_req(struct gsm_lchan *lchan, uint32_t tlli, const uint8_t *ra_id, uint8_t cause)
+{
+	struct msgb *msg;
+	struct gsm_pcu_if *pcu_prim;
+
+	msg = pcu_msgb_alloc(PCU_IF_MSG_SUSP_REQ, lchan->ts->trx->bts->nr);
+	if (!msg)
+		return -ENOMEM;
+	pcu_prim = (struct gsm_pcu_if *) msg->data;
+	pcu_prim->u.susp_req.tlli = tlli;
+	memcpy(pcu_prim->u.susp_req.ra_id, ra_id, sizeof(pcu_prim->u.susp_req.ra_id));
+	pcu_prim->u.susp_req.cause = cause;
+
+	return pcu_sock_send(&bts_gsmnet, msg);
+}
+
 static int pcu_rx_data_req(struct gsm_bts *bts, uint8_t msg_type,
-	struct gsm_pcu_if_data *data_req)
+	const struct gsm_pcu_if_data *data_req)
 {
 	uint8_t is_ptcch;
 	struct gsm_bts_trx *trx;
@@ -481,13 +523,7 @@ static int pcu_rx_data_req(struct gsm_bts *bts, uint8_t msg_type,
 
 	switch (data_req->sapi) {
 	case PCU_IF_SAPI_PCH:
-		if (msg_type == PCU_IF_MSG_PAG_REQ) {
-			/* FIXME: Add function to schedule paging request.
-			 * This might not be required, if PCU_IF_MSG_DATA_REQ
-			 * is used instead. */
-		} else {
-			paging_add_imm_ass(bts->paging_state, data_req->data, data_req->len);
-		}
+		paging_add_imm_ass(bts->paging_state, data_req->data, data_req->len);
 		break;
 	case PCU_IF_SAPI_AGCH:
 		msg = msgb_alloc(data_req->len, "pcu_agch");
@@ -543,6 +579,24 @@ static int pcu_rx_data_req(struct gsm_bts *bts, uint8_t msg_type,
 	return rc;
 }
 
+static int pcu_rx_pag_req(struct gsm_bts *bts, uint8_t msg_type,
+	const struct gsm_pcu_if_pag_req *pag_req)
+{
+	int rc = 0;
+
+	OSMO_ASSERT(msg_type == PCU_IF_MSG_PAG_REQ);
+
+	/* FIXME: Add function to schedule paging request.
+	 * At present, osmo-pcu sends paging requests in PCU_IF_MSG_DATA_REQ
+	 * messages which are processed by pcu_rx_data_req().
+	 * This code path is not triggered in practice. */
+	LOGP(DPCU, LOGL_NOTICE, "Paging request received: chan_needed=%d length=%d "
+	     "(dropping message because support for PCU_IF_MSG_PAG_REQ is not yet implemented)\n",
+	     pag_req->chan_needed, pag_req->identity_lv[0]);
+
+	return rc;
+}
+
 int pcu_tx_si13(const struct gsm_bts *bts, bool enable)
 {
 	/* the SI is per-BTS so it doesn't matter which TRX we use */
@@ -565,16 +619,20 @@ static int pcu_rx_txt_ind(struct gsm_bts *bts,
 	case PCU_VERSION:
 		LOGP(DPCU, LOGL_INFO, "OsmoPCU version %s connected\n",
 		     txt->text);
-		osmo_signal_dispatch(SS_FAIL, OSMO_EVT_PCU_VERS, txt->text);
+		oml_tx_failure_event_rep(&bts->gprs.cell.mo, NM_SEVER_CEASED, OSMO_EVT_PCU_VERS, txt->text);
 		osmo_strlcpy(bts->pcu_version, txt->text, MAX_VERSION_LENGTH);
 
+		/* patch SI3 to advertise GPRS, *if* the SI3 sent by BSC said so */
+		regenerate_si3_restoctets(bts);
+
 		if (GSM_BTS_HAS_SI(bts, SYSINFO_TYPE_13))
 			return pcu_tx_si13(bts, true);
 
 		LOGP(DPCU, LOGL_INFO, "SI13 is not available on PCU connection\n");
 		break;
 	case PCU_OML_ALERT:
-		osmo_signal_dispatch(SS_FAIL, OSMO_EVT_EXT_ALARM, txt->text);
+		oml_tx_failure_event_rep(&bts->gprs.cell.mo, NM_SEVER_INDETERMINATE, OSMO_EVT_EXT_ALARM,
+					 txt->text);
 		break;
 	default:
 		LOGP(DPCU, LOGL_ERROR, "Unknown TXT_IND type %u received\n",
@@ -586,7 +644,7 @@ static int pcu_rx_txt_ind(struct gsm_bts *bts,
 }
 
 static int pcu_rx_act_req(struct gsm_bts *bts,
-	struct gsm_pcu_if_act_req *act_req)
+	const struct gsm_pcu_if_act_req *act_req)
 {
 	struct gsm_bts_trx *trx;
 	struct gsm_lchan *lchan;
@@ -631,9 +689,11 @@ static int pcu_rx(struct gsm_network *net, uint8_t msg_type,
 
 	switch (msg_type) {
 	case PCU_IF_MSG_DATA_REQ:
-	case PCU_IF_MSG_PAG_REQ:
 		rc = pcu_rx_data_req(bts, msg_type, &pcu_prim->u.data_req);
 		break;
+	case PCU_IF_MSG_PAG_REQ:
+		rc = pcu_rx_pag_req(bts, msg_type, &pcu_prim->u.pag_req);
+		break;
 	case PCU_IF_MSG_ACT_REQ:
 		rc = pcu_rx_act_req(bts, &pcu_prim->u.act_req);
 		break;
@@ -641,7 +701,7 @@ static int pcu_rx(struct gsm_network *net, uint8_t msg_type,
 		rc = pcu_rx_txt_ind(bts, &pcu_prim->u.txt_ind);
 		break;
 	default:
-		LOGP(DPCU, LOGL_ERROR, "Received unknwon PCU msg type %d\n",
+		LOGP(DPCU, LOGL_ERROR, "Received unknown PCU msg type %d\n",
 			msg_type);
 		rc = -EINVAL;
 	}
@@ -699,13 +759,18 @@ static void pcu_sock_close(struct pcu_sock_state *state)
 	bts = llist_entry(state->net->bts_list.next, struct gsm_bts, list);
 
 	LOGP(DPCU, LOGL_NOTICE, "PCU socket has LOST connection\n");
-	osmo_signal_dispatch(SS_FAIL, OSMO_EVT_PCU_VERS, NULL);
+	oml_tx_failure_event_rep(&bts->gprs.cell.mo, NM_SEVER_MAJOR, OSMO_EVT_PCU_VERS,
+				 "PCU socket has LOST connection");
+
 	bts->pcu_version[0] = '\0';
 
 	close(bfd->fd);
 	bfd->fd = -1;
 	osmo_fd_unregister(bfd);
 
+	/* patch SI3 to remove GPRS indicator */
+	regenerate_si3_restoctets(bts);
+
 	/* re-enable the generation of ACCEPT for new connections */
 	state->listen_bfd.when |= BSC_FD_READ;
 
@@ -724,9 +789,9 @@ static void pcu_sock_close(struct pcu_sock_state *state)
 			ts = &trx->ts[j];
 			if (ts->mo.nm_state.operational == NM_OPSTATE_ENABLED
 			 && ts->pchan == GSM_PCHAN_PDCH) {
-				ts->lchan->rel_act_kind = LCHAN_REL_ACT_PCU;
+				ts->lchan[0].rel_act_kind = LCHAN_REL_ACT_PCU;
 				l1sap_chan_rel(trx,
-					gsm_lchan2chan_nr(ts->lchan));
+					gsm_lchan2chan_nr(&ts->lchan[0]));
 			}
 		}
 	}
@@ -756,14 +821,17 @@ static int pcu_sock_read(struct osmo_fd *bfd)
 		goto close;
 
 	if (rc < 0) {
-		if (errno == EAGAIN)
+		if (errno == EAGAIN) {
+			msgb_free(msg);
 			return 0;
+		}
 		goto close;
 	}
 
 	if (rc < sizeof(*pcu_prim)) {
 		LOGP(DPCU, LOGL_ERROR, "Received %d bytes on PCU Socket, but primitive size "
-			"is %lu, discarding\n", rc, sizeof(*pcu_prim));
+		     "is %zu, discarding\n", rc, sizeof(*pcu_prim));
+		msgb_free(msg);
 		return 0;
 	}
 
@@ -844,7 +912,7 @@ static int pcu_sock_cb(struct osmo_fd *bfd, unsigned int flags)
 	return rc;
 }
 
-/* accept connection comming from PCU */
+/* accept connection coming from PCU */
 static int pcu_sock_accept(struct osmo_fd *bfd, unsigned int flags)
 {
 	struct pcu_sock_state *state = (struct pcu_sock_state *)bfd->data;
@@ -932,6 +1000,8 @@ int pcu_sock_init(const char *path)
 
 	bts_gsmnet.pcu_state = state;
 
+	LOGP(DPCU, LOGL_INFO, "Started listening on PCU socket: %s\n", path);
+
 	return 0;
 }
 
