@@ -26,11 +26,13 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <ctype.h>
+#include <errno.h>
 
 #include <osmocom/core/talloc.h>
 #include <osmocom/gsm/abis_nm.h>
 #include <osmocom/gsm/protocol/gsm_04_08.h>
 #include <osmocom/vty/vty.h>
+#include <osmocom/vty/stats.h>
 #include <osmocom/vty/command.h>
 #include <osmocom/vty/logging.h>
 #include <osmocom/vty/misc.h>
@@ -227,12 +229,25 @@ DEFUN(cfg_bts_trx, cfg_bts_trx_cmd,
 	struct gsm_bts *bts = vty->index;
 	struct gsm_bts_trx *trx;
 
-	trx = gsm_bts_trx_num(bts, trx_nr);
+
+	if (trx_nr > bts->num_trx) {
+		vty_out(vty, "%% The next unused TRX number is %u%s",
+			bts->num_trx, VTY_NEWLINE);
+		return CMD_WARNING;
+	} else if (trx_nr == bts->num_trx) {
+		/* Allocate a new TRX
+		 * Remark: TRX0 was already created during gsm_bts_alloc() and
+		 * 	   initialized in bts_init(), not here.
+		 */
+		trx = gsm_bts_trx_alloc(bts);
+		if (trx)
+			bts_trx_init(trx);
+	} else
+		trx = gsm_bts_trx_num(bts, trx_nr);
+
 	if (!trx) {
-		vty_out(vty, "Unknown TRX %u. Available TRX are: 0..%u%s",
-			trx_nr, bts->num_trx - 1, VTY_NEWLINE);
-		vty_out(vty, "Hint: Check if commandline option -t matches the"
-			"number of available transceivers!%s", VTY_NEWLINE);
+		vty_out(vty, "%% Unable to allocate TRX %u%s",
+			trx_nr, VTY_NEWLINE);
 		return CMD_WARNING;
 	}
 
@@ -246,7 +261,7 @@ DEFUN(cfg_bts_trx, cfg_bts_trx_cmd,
 static void config_write_bts_single(struct vty *vty, struct gsm_bts *bts)
 {
 	struct gsm_bts_trx *trx;
-	char buf_casecnvt[256];
+	const char *sapi_buf;
 	int i;
 
 	vty_out(vty, "bts %u%s", bts->nr, VTY_NEWLINE);
@@ -262,6 +277,8 @@ static void config_write_bts_single(struct vty *vty, struct gsm_bts *bts)
 	if (bts->rtp_jitter_adaptive)
 		vty_out(vty, " adaptive");
 	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, " rtp port-range %u %u%s", bts->rtp_port_range_start,
+		bts->rtp_port_range_end, VTY_NEWLINE);
 	vty_out(vty, " paging queue-size %u%s", paging_get_queue_max(bts->paging_state),
 		VTY_NEWLINE);
 	vty_out(vty, " paging lifetime %u%s", paging_get_lifetime(bts->paging_state),
@@ -276,17 +293,17 @@ static void config_write_bts_single(struct vty *vty, struct gsm_bts *bts)
 
 	for (i = 0; i < 32; i++) {
 		if (gsmtap_sapi_mask & (1 << i)) {
-			osmo_str2lower(buf_casecnvt, get_value_string(gsmtap_sapi_names, i));
-			vty_out(vty, " gsmtap-sapi %s%s", buf_casecnvt, VTY_NEWLINE);
+			sapi_buf = osmo_str_tolower(get_value_string(gsmtap_sapi_names, i));
+			vty_out(vty, " gsmtap-sapi %s%s", sapi_buf, VTY_NEWLINE);
 		}
 	}
 	if (gsmtap_sapi_acch) {
-		osmo_str2lower(buf_casecnvt, get_value_string(gsmtap_sapi_names, GSMTAP_CHANNEL_ACCH));
-		vty_out(vty, " gsmtap-sapi %s%s", buf_casecnvt, VTY_NEWLINE);
+		sapi_buf = osmo_str_tolower(get_value_string(gsmtap_sapi_names, GSMTAP_CHANNEL_ACCH));
+		vty_out(vty, " gsmtap-sapi %s%s", sapi_buf, VTY_NEWLINE);
 	}
-	vty_out(vty, " min-qual-rach %.0f%s", bts->min_qual_rach * 10.0f,
+	vty_out(vty, " min-qual-rach %d%s", bts->min_qual_rach,
 		VTY_NEWLINE);
-	vty_out(vty, " min-qual-norm %.0f%s", bts->min_qual_norm * 10.0f,
+	vty_out(vty, " min-qual-norm %d%s", bts->min_qual_norm,
 		VTY_NEWLINE);
 	vty_out(vty, " max-ber10k-rach %u%s", bts->max_ber10k_rach,
 		VTY_NEWLINE);
@@ -294,6 +311,9 @@ static void config_write_bts_single(struct vty *vty, struct gsm_bts *bts)
 		vty_out(vty, " pcu-socket %s%s", bts->pcu.sock_path, VTY_NEWLINE);
 	if (bts->supp_meas_toa256)
 		vty_out(vty, " supp-meas-info toa256%s", VTY_NEWLINE);
+	vty_out(vty, " smscb queue-max-length %d%s", bts->smscb_queue_max_len, VTY_NEWLINE);
+	vty_out(vty, " smscb queue-target-length %d%s", bts->smscb_queue_tgt_len, VTY_NEWLINE);
+	vty_out(vty, " smscb queue-hysteresis %d%s", bts->smscb_queue_hyst, VTY_NEWLINE);
 
 	bts_model_config_write_bts(vty, bts);
 
@@ -312,7 +332,7 @@ static void config_write_bts_single(struct vty *vty, struct gsm_bts *bts)
 		vty_out(vty, "  power-ramp step-interval %d%s",
 			tpp->ramp.step_interval_sec, VTY_NEWLINE);
 		vty_out(vty, "  ms-power-control %s%s",
-			trx->ms_power_control == 0 ? "dsp" : "osmo",
+			trx->ms_pwr_ctl_soft ? "osmo" : "dsp",
 			VTY_NEWLINE);
 		vty_out(vty, "  phy %u instance %u%s", pinst->phy_link->num,
 			pinst->num, VTY_NEWLINE);
@@ -462,7 +482,7 @@ DEFUN(cfg_bts_oml_ip,
 
 #define RTP_STR "RTP parameters\n"
 
-DEFUN_HIDDEN(cfg_bts_rtp_bind_ip,
+DEFUN_DEPRECATED(cfg_bts_rtp_bind_ip,
       cfg_bts_rtp_bind_ip_cmd,
       "rtp bind-ip A.B.C.D",
       RTP_STR "RTP local bind IP Address\n" "RTP local bind IP Address\n")
@@ -486,6 +506,43 @@ DEFUN(cfg_bts_rtp_jitbuf,
 	return CMD_SUCCESS;
 }
 
+DEFUN(cfg_bts_rtp_port_range,
+	cfg_bts_rtp_port_range_cmd,
+	"rtp port-range <1-65534> <1-65534>",
+      RTP_STR "Range of local ports to use for RTP/RTCP traffic\n")
+{
+	struct gsm_bts *bts = vty->index;
+	unsigned int start;
+	unsigned int end;
+
+	start = atoi(argv[0]);
+	end = atoi(argv[1]);
+
+	if (end < start) {
+		vty_out(vty, "range end port (%u) must be greater than the range start port (%u)!%s",
+			end, start, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	if (start & 1) {
+		vty_out(vty, "range must begin at an even port number! (%u not even)%s",
+			start, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	if ((end & 1) == 0) {
+		vty_out(vty, "range must end at an odd port number! (%u not odd)%s",
+			end, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	bts->rtp_port_range_start = start;
+	bts->rtp_port_range_end = end;
+	bts->rtp_port_range_next = bts->rtp_port_range_start;
+
+	return CMD_SUCCESS;
+}
+
 #define PAG_STR "Paging related parameters\n"
 
 DEFUN(cfg_bts_paging_queue_size,
@@ -562,24 +619,24 @@ DEFUN(cfg_bts_ul_power_target, cfg_bts_ul_power_target_cmd,
 
 DEFUN(cfg_bts_min_qual_rach, cfg_bts_min_qual_rach_cmd,
 	"min-qual-rach <-100-100>",
-	"Set the minimum quality level of RACH burst to be accpeted\n"
-	"C/I level in tenth of dB\n")
+	"Set the minimum link quality level of Access Bursts to be accepted\n"
+	"C/I (Carrier-to-Interference) ratio in centiBels (10e-2 B or 10e-1 dB)\n")
 {
 	struct gsm_bts *bts = vty->index;
 
-	bts->min_qual_rach = strtof(argv[0], NULL) / 10.0f;
+	bts->min_qual_rach = atoi(argv[0]);
 
 	return CMD_SUCCESS;
 }
 
 DEFUN(cfg_bts_min_qual_norm, cfg_bts_min_qual_norm_cmd,
 	"min-qual-norm <-100-100>",
-	"Set the minimum quality level of normal burst to be accpeted\n"
-	"C/I level in tenth of dB\n")
+	"Set the minimum link quality level of Normal Bursts to be accepted\n"
+	"C/I (Carrier-to-Interference) ratio in centiBels (10e-2 B or 10e-1 dB)\n")
 {
 	struct gsm_bts *bts = vty->index;
 
-	bts->min_qual_norm = strtof(argv[0], NULL) / 10.0f;
+	bts->min_qual_norm = atoi(argv[0]);
 
 	return CMD_SUCCESS;
 }
@@ -634,6 +691,33 @@ DEFUN(cfg_bts_no_supp_meas_toa256, cfg_bts_no_supp_meas_toa256_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN(cfg_bts_smscb_max_qlen, cfg_bts_smscb_max_qlen_cmd,
+	"smscb queue-max-length <1-60>",
+	"Maximum queue length for SMSCB (CBCH) queue. In count of messages/pages (Default: 15)")
+{
+	struct gsm_bts *bts = vty->index;
+	bts->smscb_queue_max_len = atoi(argv[0]);
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_bts_smscb_tgt_qlen, cfg_bts_smscb_tgt_qlen_cmd,
+	"smscb queue-target-length <1-30>",
+	"Target queue length for SMSCB (CBCH) queue. In count of messages/pages (Default: 2)")
+{
+	struct gsm_bts *bts = vty->index;
+	bts->smscb_queue_tgt_len = atoi(argv[0]);
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_bts_smscb_qhyst, cfg_bts_smscb_qhyst_cmd,
+	"smscb queue-hysteresis <0-30>",
+	"Hysteresis for SMSCB (CBCH) queue. In count of messages/pages (Default: 2)")
+{
+	struct gsm_bts *bts = vty->index;
+	bts->smscb_queue_hyst = atoi(argv[0]);
+	return CMD_SUCCESS;
+}
+
 
 #define DB_DBM_STR 							\
 	"Unit is dB (decibels)\n"					\
@@ -705,8 +789,23 @@ DEFUN(cfg_trx_ms_power_control, cfg_trx_ms_power_control_cmd,
 	"Handled by DSP\n" "Handled by OsmoBTS\n")
 {
 	struct gsm_bts_trx *trx = vty->index;
+	bool soft = !strcmp(argv[0], "osmo");
 
-	trx->ms_power_control = argv[0][0] == 'd' ? 0 : 1;
+	if (!soft && !gsm_bts_has_feature(trx->bts, BTS_FEAT_MS_PWR_CTRL_DSP)) {
+		/* NOTE: osmo-bts-trx used to have its own (low-level) MS Power Control loop, which
+		 * has been ripped out in favour of the common implementation. Configuration files
+		 * may still contain 'dsp', so let's be tolerant and override 'dsp' by 'osmo'. */
+		if (trx->bts->variant == BTS_OSMO_TRX && vty->type == VTY_FILE) {
+			vty_out(vty, "BTS model 'osmo-bts-trx' has no DSP/HW MS Power Control support, "
+				     "consider updating your configuration file!%s", VTY_NEWLINE);
+			soft = true; /* override */
+		} else {
+			vty_out(vty, "This BTS model has no DSP/HW MS Power Control support%s", VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+	}
+
+	trx->ms_pwr_ctl_soft = soft;
 	return CMD_SUCCESS;
 }
 
@@ -750,17 +849,6 @@ static void net_dump_nmstate(struct vty *vty, struct gsm_nm_state *nms)
 		abis_nm_avail_name(nms->availability), VTY_NEWLINE);
 }
 
-static unsigned int llist_length(struct llist_head *list)
-{
-	unsigned int len = 0;
-	struct llist_head *pos;
-
-	llist_for_each(pos, list)
-		len++;
-
-	return len;
-}
-
 static void bts_dump_vty_features(struct vty *vty, struct gsm_bts *bts)
 {
 	unsigned int i;
@@ -812,13 +900,19 @@ static void bts_dump_vty(struct vty *vty, struct gsm_bts *bts)
 		bts->agch_queue.rejected_msgs, bts->agch_queue.agch_msgs,
 		bts->agch_queue.pch_msgs,
 		VTY_NEWLINE);
-	vty_out(vty, "  CBCH backlog queue length: %u%s",
-		llist_length(&bts->smscb_state.queue), VTY_NEWLINE);
+	vty_out(vty, "  CBCH queue target: %d, hysteresis: %d, maximum: %d%s",
+		bts->smscb_queue_tgt_len, bts->smscb_queue_max_len, bts->smscb_queue_hyst, VTY_NEWLINE);
+	vty_out(vty, "  CBCH backlog queue length (BASIC): %d%s",
+		bts->smscb_basic.queue_len, VTY_NEWLINE);
+	vty_out(vty, "  CBCH backlog queue length (EXTENDED): %u%s",
+		bts->smscb_extended.queue_len, VTY_NEWLINE);
 	vty_out(vty, "  Paging: queue length %d, buffer space %d%s",
 		paging_queue_length(bts->paging_state), paging_buffer_space(bts->paging_state),
 		VTY_NEWLINE);
 	vty_out(vty, "  OML Link state: %s.%s",
 		bts->oml_link ? "connected" : "disconnected", VTY_NEWLINE);
+	vty_out(vty, "  PH-RTS.ind FN advance average: %d, min: %d, max: %d%s",
+		bts_get_avg_fn_advance(bts), bts->fn_stats.min, bts->fn_stats.max, VTY_NEWLINE);
 
 	llist_for_each_entry(trx, &bts->trx_list, list) {
 		struct phy_instance *pinst = trx_phy_instance(trx);
@@ -1094,7 +1188,7 @@ static void lchan_dump_full_vty(struct vty *vty, struct gsm_lchan *lchan)
 	vty_out(vty, "  BS Power: %d dBm, MS Power: %u dBm%s",
 		lchan->ts->trx->nominal_power - lchan->ts->trx->max_power_red
 		- lchan->bs_power*2,
-		ms_pwr_dbm(lchan->ts->trx->bts->band, lchan->ms_power),
+		ms_pwr_dbm(lchan->ts->trx->bts->band, lchan->ms_power_ctrl.max),
 		VTY_NEWLINE);
 	vty_out(vty, "  Channel Mode / Codec: %s%s",
 		get_value_string(gsm48_cmode_names, lchan->tch_mode),
@@ -1411,8 +1505,8 @@ DEFUN(cfg_phy_inst, cfg_phy_inst_cmd,
 }
 
 DEFUN(cfg_phy_no_inst, cfg_phy_no_inst_cmd,
-	"no instance <0-255>"
-	NO_STR "Select a PHY instance to remove\n", "PHY Instance number\n")
+	"no instance <0-255>",
+	NO_STR "Select a PHY instance to remove\n" "PHY Instance number\n")
 {
 	int inst_nr = atoi(argv[0]);
 	struct phy_link *plink = vty->index;
@@ -1522,7 +1616,48 @@ DEFUN(no_bts_t_t_l_loopback,
 	return CMD_SUCCESS;
 }
 
-int bts_vty_init(struct gsm_bts *bts, const struct log_info *cat)
+DEFUN(logging_fltr_l1_sapi, logging_fltr_l1_sapi_cmd, "HIDDEN", "HIDDEN")
+{
+	int sapi = get_string_value(l1sap_common_sapi_names, argv[0]);
+	struct log_target *tgt = osmo_log_vty2tgt(vty);
+	uint16_t **sapi_mask;
+
+	OSMO_ASSERT(sapi >= 0);
+	if (!tgt)
+		return CMD_WARNING;
+
+	sapi_mask = (uint16_t **)&tgt->filter_data[LOG_FLT_L1_SAPI];
+
+	if (!*sapi_mask)
+		*sapi_mask = talloc(tgt, uint16_t);
+
+	OSMO_ASSERT(sapi <= 31);
+	**sapi_mask |= (1 << sapi);
+	tgt->filter_map |= (1 << LOG_FLT_L1_SAPI);
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(no_logging_fltr_l1_sapi, no_logging_fltr_l1_sapi_cmd, "HIDDEN", "HIDDEN")
+{
+	int sapi = get_string_value(l1sap_common_sapi_names, argv[0]);
+	struct log_target *tgt = osmo_log_vty2tgt(vty);
+	uint16_t *sapi_mask;
+
+	OSMO_ASSERT(sapi >= 0);
+	if (!tgt)
+		return CMD_WARNING;
+	if (!tgt->filter_data[LOG_FLT_L1_SAPI])
+		return CMD_SUCCESS;
+
+	OSMO_ASSERT(sapi <= 31);
+	sapi_mask = (uint16_t *)tgt->filter_data[LOG_FLT_L1_SAPI];
+	*sapi_mask &= ~(1 << sapi);
+
+	return CMD_SUCCESS;
+}
+
+int bts_vty_init(struct gsm_bts *bts)
 {
 	cfg_trx_gsmtap_sapi_cmd.string = vty_cmd_string_from_valstr(bts, gsmtap_sapi_names,
 						"gsmtap-sapi (",
@@ -1538,14 +1673,31 @@ int bts_vty_init(struct gsm_bts *bts, const struct log_info *cat)
 						NO_STR "GSMTAP SAPI\n",
 						"\n", "", 0);
 
+	logging_fltr_l1_sapi_cmd.string = vty_cmd_string_from_valstr(bts, l1sap_common_sapi_names,
+						"logging filter l1-sapi (",
+						"|", ")", VTY_DO_LOWER);
+	logging_fltr_l1_sapi_cmd.doc = vty_cmd_string_from_valstr(bts, l1sap_common_sapi_names,
+						LOGGING_STR FILTER_STR "L1 SAPI\n",
+						"\n", "", 0);
+
+	no_logging_fltr_l1_sapi_cmd.string = vty_cmd_string_from_valstr(bts, l1sap_common_sapi_names,
+						"no logging filter l1-sapi (",
+						"|", ")", VTY_DO_LOWER);
+	no_logging_fltr_l1_sapi_cmd.doc = vty_cmd_string_from_valstr(bts, l1sap_common_sapi_names,
+						NO_STR LOGGING_STR FILTER_STR "L1 SAPI\n",
+						"\n", "", 0);
+
 	install_element_ve(&show_bts_cmd);
 	install_element_ve(&show_trx_cmd);
 	install_element_ve(&show_ts_cmd);
 	install_element_ve(&show_lchan_cmd);
 	install_element_ve(&show_lchan_summary_cmd);
+	install_element_ve(&logging_fltr_l1_sapi_cmd);
+	install_element_ve(&no_logging_fltr_l1_sapi_cmd);
 
-	logging_vty_add_cmds(cat);
+	logging_vty_add_cmds();
 	osmo_talloc_vty_add_cmds();
+	osmo_stats_vty_add_cmds();
 
 	install_node(&bts_node, config_write_bts);
 	install_element(CONFIG_NODE, &cfg_bts_cmd);
@@ -1554,6 +1706,7 @@ int bts_vty_init(struct gsm_bts *bts, const struct log_info *cat)
 	install_element(BTS_NODE, &cfg_bts_oml_ip_cmd);
 	install_element(BTS_NODE, &cfg_bts_rtp_bind_ip_cmd);
 	install_element(BTS_NODE, &cfg_bts_rtp_jitbuf_cmd);
+	install_element(BTS_NODE, &cfg_bts_rtp_port_range_cmd);
 	install_element(BTS_NODE, &cfg_bts_band_cmd);
 	install_element(BTS_NODE, &cfg_description_cmd);
 	install_element(BTS_NODE, &cfg_no_description_cmd);
@@ -1568,6 +1721,9 @@ int bts_vty_init(struct gsm_bts *bts, const struct log_info *cat)
 	install_element(BTS_NODE, &cfg_bts_pcu_sock_cmd);
 	install_element(BTS_NODE, &cfg_bts_supp_meas_toa256_cmd);
 	install_element(BTS_NODE, &cfg_bts_no_supp_meas_toa256_cmd);
+	install_element(BTS_NODE, &cfg_bts_smscb_max_qlen_cmd);
+	install_element(BTS_NODE, &cfg_bts_smscb_tgt_qlen_cmd);
+	install_element(BTS_NODE, &cfg_bts_smscb_qhyst_cmd);
 
 	install_element(BTS_NODE, &cfg_trx_gsmtap_sapi_cmd);
 	install_element(BTS_NODE, &cfg_trx_no_gsmtap_sapi_cmd);
