@@ -50,7 +50,6 @@
 #include <osmo-bts/measurement.h>
 #include <osmo-bts/pcu_if.h>
 #include <osmo-bts/handover.h>
-#include <osmo-bts/cbch.h>
 #include <osmo-bts/bts_model.h>
 #include <osmo-bts/l1sap.h>
 #include <osmo-bts/msg_utils.h>
@@ -384,10 +383,12 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 		sapi = GsmL1_Sapi_Sdcch;
 	} else if (L1SAP_IS_CHAN_BCCH(chan_nr)) {
 		sapi = GsmL1_Sapi_Bcch;
+	} else if (L1SAP_IS_CHAN_CBCH(chan_nr)) {
+		sapi = GsmL1_Sapi_Cbch;
 	} else if (L1SAP_IS_CHAN_AGCH_PCH(chan_nr)) {
 		/* The sapi depends on DSP configuration, not
 		 * on the actual SYSTEM INFORMATION 3. */
-		u8BlockNbr = L1SAP_FN2CCCHBLOCK(u32Fn);
+		u8BlockNbr = l1sap_fn2ccch_block(u32Fn);
 		if (u8BlockNbr >= num_agch(trx, "PH-DATA-REQ"))
 			sapi = GsmL1_Sapi_Pch;
 		else
@@ -700,6 +701,9 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 	case GsmL1_Sapi_Bcch:
 		cbits = 0x10;
 		break;
+	case GsmL1_Sapi_Cbch:
+		cbits = 0xc8 >> 3; /* Osmocom extension for CBCH via L1SAP */
+		break;
 	case GsmL1_Sapi_Sacch:
 		switch(pchan) {
 		case GSM_PCHAN_TCH_F:
@@ -709,9 +713,11 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 			cbits = 0x02 + subCh;
 			break;
 		case GSM_PCHAN_CCCH_SDCCH4:
+		case GSM_PCHAN_CCCH_SDCCH4_CBCH:
 			cbits = 0x04 + subCh;
 			break;
 		case GSM_PCHAN_SDCCH8_SACCH8C:
+		case GSM_PCHAN_SDCCH8_SACCH8C_CBCH:
 			cbits = 0x08 + subCh;
 			break;
 		default:
@@ -723,9 +729,11 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 	case GsmL1_Sapi_Sdcch:
 		switch(pchan) {
 		case GSM_PCHAN_CCCH_SDCCH4:
+		case GSM_PCHAN_CCCH_SDCCH4_CBCH:
 			cbits = 0x04 + subCh;
 			break;
 		case GSM_PCHAN_SDCCH8_SACCH8C:
+		case GSM_PCHAN_SDCCH8_SACCH8C_CBCH:
 			cbits = 0x08 + subCh;
 			break;
 		default:
@@ -787,6 +795,45 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 	return (cbits << 3) | u8Tn;
 }
 
+static const enum l1sap_common_sapi common_sapi_by_sapi_t[] = {
+	[GsmL1_Sapi_Idle]	= L1SAP_COMMON_SAPI_IDLE,
+	[GsmL1_Sapi_Fcch]	= L1SAP_COMMON_SAPI_FCCH,
+	[GsmL1_Sapi_Sch]	= L1SAP_COMMON_SAPI_SCH,
+	[GsmL1_Sapi_Sacch]	= L1SAP_COMMON_SAPI_SACCH,
+	[GsmL1_Sapi_Sdcch]	= L1SAP_COMMON_SAPI_SDCCH,
+	[GsmL1_Sapi_Bcch]	= L1SAP_COMMON_SAPI_BCCH,
+	[GsmL1_Sapi_Pch]	= L1SAP_COMMON_SAPI_PCH,
+	[GsmL1_Sapi_Agch]	= L1SAP_COMMON_SAPI_AGCH,
+	[GsmL1_Sapi_Cbch]	= L1SAP_COMMON_SAPI_CBCH,
+	[GsmL1_Sapi_Rach]	= L1SAP_COMMON_SAPI_RACH,
+	[GsmL1_Sapi_TchF]	= L1SAP_COMMON_SAPI_TCH_F,
+	[GsmL1_Sapi_FacchF]	= L1SAP_COMMON_SAPI_FACCH_F,
+	[GsmL1_Sapi_TchH]	= L1SAP_COMMON_SAPI_TCH_H,
+	[GsmL1_Sapi_FacchH]	= L1SAP_COMMON_SAPI_FACCH_H,
+	[GsmL1_Sapi_Nch]	= L1SAP_COMMON_SAPI_NCH,
+	[GsmL1_Sapi_Pdtch]	= L1SAP_COMMON_SAPI_PDTCH,
+	[GsmL1_Sapi_Pacch]	= L1SAP_COMMON_SAPI_PACCH,
+	[GsmL1_Sapi_Pbcch]	= L1SAP_COMMON_SAPI_PBCCH,
+	[GsmL1_Sapi_Pagch]	= L1SAP_COMMON_SAPI_PAGCH,
+	[GsmL1_Sapi_Ppch]	= L1SAP_COMMON_SAPI_PPCH,
+	[GsmL1_Sapi_Pnch]	= L1SAP_COMMON_SAPI_PNCH,
+	[GsmL1_Sapi_Ptcch]	= L1SAP_COMMON_SAPI_PTCCH,
+	[GsmL1_Sapi_Prach]	= L1SAP_COMMON_SAPI_PRACH,
+};
+
+static enum l1sap_common_sapi get_common_sapi(GsmL1_Sapi_t sapi)
+{
+	if (sapi >= GsmL1_Sapi_NUM)
+		return L1SAP_COMMON_SAPI_UNKNOWN;
+	return common_sapi_by_sapi_t[sapi];
+}
+
+static void set_log_ctx_sapi(GsmL1_Sapi_t sapi)
+{
+	l1sap_log_ctx_sapi = get_common_sapi(sapi);
+	log_set_context(LOG_CTX_L1_SAPI, &l1sap_log_ctx_sapi);
+}
+
 static int handle_ph_readytosend_ind(struct lc15l1_hdl *fl1,
 				     GsmL1_PhReadyToSendInd_t *rts_ind,
 				     struct msgb *l1p_msg)
@@ -803,6 +850,8 @@ static int handle_ph_readytosend_ind(struct lc15l1_hdl *fl1,
 	uint8_t chan_nr, link_id;
 	uint32_t fn;
 
+	set_log_ctx_sapi(rts_ind->sapi);
+
 	/* check if primitive should be handled by common part */
 	chan_nr = chan_nr_by_sapi(&trx->ts[rts_ind->u8Tn], rts_ind->sapi,
 		rts_ind->subCh, rts_ind->u8Tn, rts_ind->u32Fn);
@@ -863,10 +912,6 @@ static int handle_ph_readytosend_ind(struct lc15l1_hdl *fl1,
 	case GsmL1_Sapi_Prach:
 		goto empty_frame;
 		break;
-	case GsmL1_Sapi_Cbch:
-		/* get them from bts->si_buf[] */
-		bts_cbch_get(bts, msu_param->u8Buffer, &g_time);
-		break;
 	default:
 		memcpy(msu_param->u8Buffer, fill_frame, GSM_MACBLOCK_LEN);
 		break;
@@ -929,6 +974,8 @@ static int handle_ph_data_ind(struct lc15l1_hdl *fl1, GsmL1_PhDataInd_t *data_in
 	int rc = 0;
 	int8_t rssi;
 
+	set_log_ctx_sapi(data_ind->sapi);
+
 	chan_nr = chan_nr_by_sapi(&trx->ts[data_ind->u8Tn], data_ind->sapi,
 		data_ind->subCh, data_ind->u8Tn, data_ind->u32Fn);
 	fn = data_ind->u32Fn;
@@ -993,18 +1040,12 @@ static int handle_ph_ra_ind(struct lc15l1_hdl *fl1, GsmL1_PhRaInd_t *ra_ind,
 			    struct msgb *l1p_msg)
 {
 	struct gsm_bts_trx *trx = lc15l1_hdl_trx(fl1);
-	struct gsm_bts *bts = trx->bts;
 	struct gsm_lchan *lchan;
 	struct osmo_phsap_prim *l1sap;
 	int rc;
 	struct ph_rach_ind_param rach_ind_param;
 
-	/* FIXME: this should be deprecated/obsoleted as it bypasses rach.busy counting */
-	if (ra_ind->measParam.fLinkQuality < bts->min_qual_rach) {
-		msgb_free(l1p_msg);
-		return 0;
-	}
-
+	set_log_ctx_sapi(ra_ind->sapi);
 	dump_meas_res(LOGL_DEBUG, &ra_ind->measParam);
 
 	if ((ra_ind->msgUnitParam.u8Size != 1) &&
@@ -1025,13 +1066,14 @@ static int handle_ph_ra_ind(struct lc15l1_hdl *fl1, GsmL1_PhRaInd_t *ra_ind,
 		.rssi = (int8_t) ra_ind->measParam.fRssi,
 		.ber10k = (unsigned int) (ra_ind->measParam.fBer * 10000.0),
 		.acc_delay_256bits = ra_ind->measParam.i16BurstTiming * 64,
+		.lqual_cb = (int16_t) ra_ind->measParam.fLinkQuality * 10, /* centiBels */
 	};
 
 	lchan = l1if_hLayer_to_lchan(trx, (uint32_t)ra_ind->hLayer2);
 	if (!lchan || lchan->ts->pchan == GSM_PCHAN_CCCH ||
 	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4 ||
 	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH)
-		rach_ind_param.chan_nr = 0x88;
+		rach_ind_param.chan_nr = RSL_CHAN_RACH;
 	else
 		rach_ind_param.chan_nr = gsm_lchan2chan_nr(lchan);
 
