@@ -43,7 +43,6 @@
 #include <osmo-bts/logging.h>
 #include <osmo-bts/l1sap.h>
 #include <osmo-bts/handover.h>
-#include <osmo-bts/cbch.h>
 
 #include "l1_if.h"
 #include "l1_oml.h"
@@ -357,6 +356,9 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 	case cOCTVC1_GSM_SAPI_ENUM_BCCH:
 		cbits = 0x10;
 		break;
+	case cOCTVC1_GSM_SAPI_ENUM_CBCH:
+		cbits = 0xc8 >> 3; /* Osmocom extension for CBCH via L1SAP */
+		break;
 	case cOCTVC1_GSM_SAPI_ENUM_SACCH:
 		switch (pchan) {
 		case GSM_PCHAN_TCH_F:
@@ -515,6 +517,8 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 		sapi = cOCTVC1_GSM_SAPI_ENUM_SDCCH;
 	} else if (L1SAP_IS_CHAN_BCCH(chan_nr)) {
 		sapi = cOCTVC1_GSM_SAPI_ENUM_BCCH;
+	} else if (L1SAP_IS_CHAN_CBCH(chan_nr)) {
+		sapi = cOCTVC1_GSM_SAPI_ENUM_CBCH;
 	} else if (L1SAP_IS_CHAN_AGCH_PCH(chan_nr)) {
 		sapi = cOCTVC1_GSM_SAPI_ENUM_PCH_AGCH;
 	} else {
@@ -788,6 +792,11 @@ int bts_model_init(struct gsm_bts *bts)
 	return 0;
 }
 
+int bts_model_trx_init(struct gsm_bts_trx *trx)
+{
+	return 0;
+}
+
 /***********************************************************************
  * handling of messages coming up from PHY
  ***********************************************************************/
@@ -929,6 +938,47 @@ static int handle_mph_time_ind(struct octphy_hdl *fl1, uint8_t trx_id, uint32_t
 	return 0;
 }
 
+/* octv1_gsm_api.h does not have an end marker for CTVC1_GSM_SAPI_ENUM */
+#define _OCTVC1_GSM_SAPI_ENUM_LENGTH (cOCTVC1_GSM_SAPI_ENUM_PRACH + 1)
+
+static const enum l1sap_common_sapi common_sapi_by_oct_sapi[] = {
+	[cOCTVC1_GSM_SAPI_ENUM_IDLE]		= L1SAP_COMMON_SAPI_IDLE,
+	[cOCTVC1_GSM_SAPI_ENUM_FCCH]		= L1SAP_COMMON_SAPI_FCCH,
+	[cOCTVC1_GSM_SAPI_ENUM_SCH]		= L1SAP_COMMON_SAPI_SCH,
+	[cOCTVC1_GSM_SAPI_ENUM_SACCH]		= L1SAP_COMMON_SAPI_SACCH,
+	[cOCTVC1_GSM_SAPI_ENUM_SDCCH]		= L1SAP_COMMON_SAPI_SDCCH,
+	[cOCTVC1_GSM_SAPI_ENUM_BCCH]		= L1SAP_COMMON_SAPI_BCCH,
+	[cOCTVC1_GSM_SAPI_ENUM_PCH_AGCH]	= L1SAP_COMMON_SAPI_PCH,
+	[cOCTVC1_GSM_SAPI_ENUM_CBCH]		= L1SAP_COMMON_SAPI_CBCH,
+	[cOCTVC1_GSM_SAPI_ENUM_RACH]		= L1SAP_COMMON_SAPI_RACH,
+	[cOCTVC1_GSM_SAPI_ENUM_TCHF]		= L1SAP_COMMON_SAPI_TCH_F,
+	[cOCTVC1_GSM_SAPI_ENUM_FACCHF]		= L1SAP_COMMON_SAPI_FACCH_F,
+	[cOCTVC1_GSM_SAPI_ENUM_TCHH]		= L1SAP_COMMON_SAPI_TCH_H,
+	[cOCTVC1_GSM_SAPI_ENUM_FACCHH]		= L1SAP_COMMON_SAPI_FACCH_H,
+	[cOCTVC1_GSM_SAPI_ENUM_NCH]		= L1SAP_COMMON_SAPI_NCH,
+	[cOCTVC1_GSM_SAPI_ENUM_PDTCH]		= L1SAP_COMMON_SAPI_PDTCH,
+	[cOCTVC1_GSM_SAPI_ENUM_PACCH]		= L1SAP_COMMON_SAPI_PACCH,
+	[cOCTVC1_GSM_SAPI_ENUM_PBCCH]		= L1SAP_COMMON_SAPI_PBCCH,
+	[cOCTVC1_GSM_SAPI_ENUM_PAGCH]		= L1SAP_COMMON_SAPI_PAGCH,
+	[cOCTVC1_GSM_SAPI_ENUM_PPCH]		= L1SAP_COMMON_SAPI_PPCH,
+	[cOCTVC1_GSM_SAPI_ENUM_PNCH]		= L1SAP_COMMON_SAPI_PNCH,
+	[cOCTVC1_GSM_SAPI_ENUM_PTCCH]		= L1SAP_COMMON_SAPI_PTCCH,
+	[cOCTVC1_GSM_SAPI_ENUM_PRACH]		= L1SAP_COMMON_SAPI_PRACH,
+};
+
+static enum l1sap_common_sapi get_common_sapi(tOCT_UINT8 sapi)
+{
+	if (sapi >= _OCTVC1_GSM_SAPI_ENUM_LENGTH)
+		return L1SAP_COMMON_SAPI_UNKNOWN;
+	return common_sapi_by_oct_sapi[sapi];
+}
+
+static void set_log_ctx_sapi(tOCT_UINT8 sapi)
+{
+	l1sap_log_ctx_sapi = get_common_sapi(sapi);
+	log_set_context(LOG_CTX_L1_SAPI, &l1sap_log_ctx_sapi);
+}
+
 static int handle_ph_readytosend_ind(struct octphy_hdl *fl1,
 	tOCTVC1_GSM_MSG_TRX_LOGICAL_CHANNEL_READY_TO_SEND_INDICATION_EVT *evt,
 	struct msgb *l1p_msg)
@@ -946,7 +996,9 @@ static int handle_ph_readytosend_ind(struct octphy_hdl *fl1,
 	struct msgb *resp_msg;
 	tOCTVC1_GSM_MSG_TRX_REQUEST_LOGICAL_CHANNEL_DATA_CMD *data_req;
 
-	/* Retrive the data */
+	set_log_ctx_sapi(evt->LchId.bySAPI);
+
+	/* Retrieve the data */
 	fn = evt->ulFrameNumber;
 	ts_num = (uint8_t) evt->LchId.byTimeslotNb;
 	sc = (uint8_t) evt->LchId.bySubChannelNb;
@@ -1017,10 +1069,6 @@ static int handle_ph_readytosend_ind(struct octphy_hdl *fl1,
 		    	(g_time.t1 << 7) | (g_time.t2 << 2) | (t3p >> 1);
 		data_req->Data.abyDataContent[3] = (t3p & 1);
 		break;
-	case cOCTVC1_GSM_SAPI_ENUM_CBCH:
-		rc = bts_cbch_get(bts, data_req->Data.abyDataContent, &g_time);
-		data_req->Data.ulDataLength = 23;   /* GSM MAX BLK SIZE */
-		break;
 	case cOCTVC1_GSM_SAPI_ENUM_PRACH:
 #if 0
 		/* in case we decide to send an empty frame... */
@@ -1074,6 +1122,8 @@ static int handle_ph_data_ind(struct octphy_hdl *fl1,
 	uint8_t ts_num = (uint8_t) data_ind->LchId.byTimeslotNb;
 	uint8_t sc = (uint8_t) data_ind->LchId.bySubChannelNb;
 
+	set_log_ctx_sapi(data_ind->LchId.bySAPI);
+
 	/* Need to combine two 16bit MSB and LSB to form 32bit FN */
 	fn = data_ind->Data.ulFrameNumber;
 
@@ -1144,7 +1194,7 @@ static int handle_ph_data_ind(struct octphy_hdl *fl1,
 	/* burst timing  in 1x but PCU is expecting 4X */
 	l1sap->u.data.ta_offs_256bits = data_ind->MeasurementInfo.sBurstTiming4x*64;
 	snr = data_ind->MeasurementInfo.sSNRDb;
-	/* FIXME: better converion formulae for SnR -> C / I?
+	/* FIXME: better conversion formulae for SnR -> C / I?
 	l1sap->u.data.lqual_cb = (snr ? snr : (snr - 65536)) * 10 / 256;
 	LOGP(DL1C, LOGL_ERROR, "SnR: raw %d, computed %d\n", snr, l1sap->u.data.lqual_cb);
 	*/
@@ -1166,11 +1216,13 @@ static int handle_ph_rach_ind(struct octphy_hdl *fl1,
 	int rc;
 	struct ph_rach_ind_param rach_ind_param;
 
+	set_log_ctx_sapi(ra_ind->LchId.bySAPI);
+
 	dump_meas_res(LOGL_DEBUG, &ra_ind->MeasurementInfo);
 
 	if (ra_ind->ulMsgLength != 1) {
 		LOGPFN(DL1C, LOGL_ERROR, ra_ind->ulFrameNumber,
-			"Rx PH-RACH.ind has lenghth %d > 1\n", ra_ind->ulMsgLength);
+			"Rx PH-RACH.ind has length %d > 1\n", ra_ind->ulMsgLength);
 		msgb_free(l1p_msg);
 		return 0;
 	}
@@ -1190,7 +1242,7 @@ static int handle_ph_rach_ind(struct octphy_hdl *fl1,
 
 	if (ra_ind->LchId.bySubChannelNb == cOCTVC1_GSM_ID_SUB_CHANNEL_NB_ENUM_ALL &&
 	    ra_ind->LchId.bySAPI == cOCTVC1_GSM_SAPI_ENUM_RACH) {
-		rach_ind_param.chan_nr = 0x88;
+		rach_ind_param.chan_nr = RSL_CHAN_RACH;
 	} else {
 		struct gsm_lchan *lchan = get_lchan_by_lchid(trx, &ra_ind->LchId);
 		rach_ind_param.chan_nr = gsm_lchan2chan_nr(lchan);
@@ -1629,7 +1681,7 @@ static int rx_octphy_msg(struct msgb *msg)
 	}
 
 	/* we first need to decode the common OCTPKT header and dispatch
-	 * based on contrl (command/resp) or data (event=indication) */
+	 * based on control (command/resp) or data (event=indication) */
 	switch (format) {
 	case cOCTVOCNET_PKT_FORMAT_CTRL:
 		ctlh = (tOCTVOCNET_PKT_CTL_HEADER *) (msg->l1h + 4);
@@ -1659,6 +1711,7 @@ void bts_model_phy_link_set_defaults(struct phy_link *plink)
 	plink->u.octphy.rf_port_index = 0;
 	plink->u.octphy.rx_gain_db = 70;
 	plink->u.octphy.tx_atten_db = 0;
+	plink->u.octphy.over_sample_16x = true;
 }
 
 void bts_model_phy_instance_set_defaults(struct phy_instance *pinst)
