@@ -506,9 +506,9 @@ static int set_ciph_compl_cb(struct octphy_hdl *fl1, struct msgb *resp, void *da
 	 * sub-channel, only th request contains this information :( */
 	lchan = &ts->lchan[(unsigned long) data];
 
-	/* TODO: This state machine should be shared accross BTS models? */
+	/* TODO: This state machine should be shared across BTS models? */
 	switch (lchan->ciph_state) {
-	case LCHAN_CIPH_RX_REQ:	
+	case LCHAN_CIPH_RX_REQ:
 		lchan->ciph_state = LCHAN_CIPH_RX_CONF;
 		break;
 	case LCHAN_CIPH_RX_CONF_TX_REQ:
@@ -1191,7 +1191,7 @@ static int app_info_compl_cb(struct octphy_hdl *fl1h, struct msgb *resp,
 
 		if (!no_fw_check) {
 			LOGP(DL1C, LOGL_ERROR,
-			     "use option -I to override the check (not recommened)\n");
+			     "use option -I to override the check (not recommended)\n");
 			LOGP(DL1C, LOGL_ERROR,
 			     "exiting...\n");
 			exit(1);
@@ -1346,7 +1346,7 @@ int l1if_trx_open(struct gsm_bts_trx *trx)
 	if (plink->u.octphy.tx_atten_flag) {
 		oc->RfConfig.ulTxAttndB = plink->u.octphy.tx_atten_db;
 	} else {
-		/* Take the Tx Attn received in set radio attribures
+		/* Take the Tx Attn received in set radio attributes
 		 * x4 is for the value in db */
 		oc->RfConfig.ulTxAttndB = (trx->max_power_red) << 2;
 	}
@@ -1375,6 +1375,57 @@ int l1if_trx_open(struct gsm_bts_trx *trx)
 	return l1if_req_compl(fl1h, msg, trx_open_compl_cb, NULL);
 }
 
+#if OCTPHY_USE_16X_OVERSAMPLING == 1
+static int over_sample_16x_modif_compl_cb(struct octphy_hdl *fl1,
+					  struct msgb *resp, void *data)
+{
+	tOCTVC1_GSM_MSG_OVERSAMPLE_SELECT_16X_MODIFY_RSP *mcr =
+	    (tOCTVC1_GSM_MSG_OVERSAMPLE_SELECT_16X_MODIFY_RSP*) resp->l2h;
+
+	/* in a completion call-back, we take msgb ownership and must
+	 * release it before returning */
+
+	mOCTVC1_GSM_MSG_OVERSAMPLE_SELECT_16X_MODIFY_RSP_SWAP(mcr);
+
+	LOGP(DL1C, LOGL_INFO, "Rx OVER-SAMPLE-16x-MODIFY.conf\n");
+
+	msgb_free(resp);
+
+	return 0;
+}
+
+static int l1if_over_sample_16x_modif(struct gsm_bts_trx *trx)
+{
+	/* NOTE: The 16x oversampling mode should always be enabled. Single-
+	 * TRX operation will work with standard 4x oversampling, but multi-
+	 * TRX requires 16x oversampling */
+
+	struct phy_instance *pinst = trx_phy_instance(trx);
+	struct phy_link *plink = pinst->phy_link;
+	struct octphy_hdl *fl1h = pinst->phy_link->u.octphy.hdl;
+	struct msgb *msg = l1p_msgb_alloc();
+	tOCTVC1_GSM_MSG_OVERSAMPLE_SELECT_16X_MODIFY_CMD *mc;
+
+	mc = (tOCTVC1_GSM_MSG_OVERSAMPLE_SELECT_16X_MODIFY_CMD*) msgb_put(msg,
+									  sizeof
+									  (*mc));
+	mOCTVC1_GSM_MSG_OVERSAMPLE_SELECT_16X_MODIFY_CMD_DEF(mc);
+	l1if_fill_msg_hdr(&mc->Header, msg, fl1h, cOCTVC1_MSG_TYPE_COMMAND,
+			  cOCTVC1_GSM_MSG_OVERSAMPLE_SELECT_16X_MODIFY_CID);
+
+	if (plink->u.octphy.over_sample_16x == true)
+		mc->ulOversample16xEnableFlag = 1;
+	else
+		mc->ulOversample16xEnableFlag = 0;
+
+	mOCTVC1_GSM_MSG_OVERSAMPLE_SELECT_16X_MODIFY_CMD_SWAP(mc);
+
+	LOGP(DL1C, LOGL_INFO, "Tx OVER-SAMPLE-16x-MODIF.req\n");
+
+	return l1if_req_compl(fl1h, msg, over_sample_16x_modif_compl_cb, 0);
+}
+#endif
+
 uint32_t trx_get_hlayer1(struct gsm_bts_trx * trx)
 {
 	return 0;
@@ -1393,6 +1444,10 @@ static int trx_init(struct gsm_bts_trx *trx)
 	l1if_check_app_version(trx);
 	l1if_check_app_sys_version(trx);
 
+#if OCTPHY_USE_16X_OVERSAMPLING == 1
+	l1if_over_sample_16x_modif(trx);
+#endif
+
 	return l1if_trx_open(trx);
 }
 
@@ -1550,7 +1605,7 @@ static int ts_connect_cb(struct octphy_hdl *fl1, struct msgb *resp, void *data)
 
 	msgb_free(resp);
 
-	cb_ts_connected(ts);
+	cb_ts_connected(ts, 0);
 
 	return 0;
 }
@@ -1750,9 +1805,10 @@ int bts_model_ts_disconnect(struct gsm_bts_trx_ts *ts)
 	return l1if_req_compl(fl1h, msg, ts_disconnect_cb, NULL);
 }
 
-int bts_model_ts_connect(struct gsm_bts_trx_ts *ts,
+void bts_model_ts_connect(struct gsm_bts_trx_ts *ts,
 			 enum gsm_phys_chan_config as_pchan)
 {
+	int rc;
 	if (as_pchan == GSM_PCHAN_TCH_F_PDCH
 	    || as_pchan == GSM_PCHAN_TCH_F_TCH_H_PDCH) {
 		LOGP(DL1C, LOGL_ERROR,
@@ -1760,8 +1816,10 @@ int bts_model_ts_connect(struct gsm_bts_trx_ts *ts,
 		     " expected a specific pchan instead\n",
 		     gsm_ts_and_pchan_name(ts), gsm_pchan_name(as_pchan));
 		exit(1);
-		return -EINVAL;
+		return;
 	}
 
-	return ts_connect_as(ts, as_pchan, ts_connect_cb, NULL);
+	rc = ts_connect_as(ts, as_pchan, ts_connect_cb, NULL);
+	if (rc)
+		cb_ts_connected(ts, rc);
 }
