@@ -46,7 +46,6 @@
 #include <osmo-bts/measurement.h>
 #include <osmo-bts/pcu_if.h>
 #include <osmo-bts/handover.h>
-#include <osmo-bts/cbch.h>
 #include <osmo-bts/bts_model.h>
 #include <osmo-bts/l1sap.h>
 #include <osmo-bts/msg_utils.h>
@@ -381,11 +380,13 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 		sapi = GsmL1_Sapi_Sdcch;
 	} else if (L1SAP_IS_CHAN_BCCH(chan_nr)) {
 		sapi = GsmL1_Sapi_Bcch;
+	} else if (L1SAP_IS_CHAN_CBCH(chan_nr)) {
+		sapi = GsmL1_Sapi_Cbch;
 	} else if (L1SAP_IS_CHAN_AGCH_PCH(chan_nr)) {
 		/* The sapi depends on DSP configuration, not
 		 * on the actual SYSTEM INFORMATION 3. */
-		u8BlockNbr = L1SAP_FN2CCCHBLOCK(u32Fn);
-		if (u8BlockNbr >= 1)
+		u8BlockNbr = l1sap_fn2ccch_block(u32Fn);
+		if (u8BlockNbr >= num_agch(trx, "PH-DATA-REQ"))
 			sapi = GsmL1_Sapi_Pch;
 		else
 			sapi = GsmL1_Sapi_Agch;
@@ -548,7 +549,11 @@ static int ph_tch_req(struct gsm_bts_trx *trx, struct msgb *msg,
 		empty_req_from_l1sap(l1p, fl1, u8Tn, u32Fn, sapi, subCh, u8BlockNbr);
 	}
 	/* send message to DSP's queue */
-	osmo_wqueue_enqueue(&fl1->write_q[MQ_L1_WRITE], nmsg);
+	if (osmo_wqueue_enqueue(&fl1->write_q[MQ_L1_WRITE], nmsg) != 0) {
+		LOGPFN(DL1P, LOGL_ERROR, u32Fn, "MQ_L1_WRITE queue full. Dropping msg.\n");
+		msgb_free(nmsg);
+		return -ENOBUFS;
+	}
 	if (dtx_is_first_p1(lchan))
 		dtx_dispatch(lchan, E_FIRST);
 	else
@@ -697,6 +702,9 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 	case GsmL1_Sapi_Bcch:
 		cbits = 0x10;
 		break;
+	case GsmL1_Sapi_Cbch:
+		cbits = 0xc8 >> 3; /* Osmocom extension for CBCH via L1SAP */
+		break;
 	case GsmL1_Sapi_Sacch:
 		switch(pchan) {
 		case GSM_PCHAN_TCH_F:
@@ -706,9 +714,11 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 			cbits = 0x02 + subCh;
 			break;
 		case GSM_PCHAN_CCCH_SDCCH4:
+		case GSM_PCHAN_CCCH_SDCCH4_CBCH:
 			cbits = 0x04 + subCh;
 			break;
 		case GSM_PCHAN_SDCCH8_SACCH8C:
+		case GSM_PCHAN_SDCCH8_SACCH8C_CBCH:
 			cbits = 0x08 + subCh;
 			break;
 		default:
@@ -720,9 +730,11 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 	case GsmL1_Sapi_Sdcch:
 		switch(pchan) {
 		case GSM_PCHAN_CCCH_SDCCH4:
+		case GSM_PCHAN_CCCH_SDCCH4_CBCH:
 			cbits = 0x04 + subCh;
 			break;
 		case GSM_PCHAN_SDCCH8_SACCH8C:
+		case GSM_PCHAN_SDCCH8_SACCH8C_CBCH:
 			cbits = 0x08 + subCh;
 			break;
 		default:
@@ -784,6 +796,45 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 	return (cbits << 3) | u8Tn;
 }
 
+static const enum l1sap_common_sapi common_sapi_by_sapi_t[] = {
+	[GsmL1_Sapi_Idle]	= L1SAP_COMMON_SAPI_IDLE,
+	[GsmL1_Sapi_Fcch]	= L1SAP_COMMON_SAPI_FCCH,
+	[GsmL1_Sapi_Sch]	= L1SAP_COMMON_SAPI_SCH,
+	[GsmL1_Sapi_Sacch]	= L1SAP_COMMON_SAPI_SACCH,
+	[GsmL1_Sapi_Sdcch]	= L1SAP_COMMON_SAPI_SDCCH,
+	[GsmL1_Sapi_Bcch]	= L1SAP_COMMON_SAPI_BCCH,
+	[GsmL1_Sapi_Pch]	= L1SAP_COMMON_SAPI_PCH,
+	[GsmL1_Sapi_Agch]	= L1SAP_COMMON_SAPI_AGCH,
+	[GsmL1_Sapi_Cbch]	= L1SAP_COMMON_SAPI_CBCH,
+	[GsmL1_Sapi_Rach]	= L1SAP_COMMON_SAPI_RACH,
+	[GsmL1_Sapi_TchF]	= L1SAP_COMMON_SAPI_TCH_F,
+	[GsmL1_Sapi_FacchF]	= L1SAP_COMMON_SAPI_FACCH_F,
+	[GsmL1_Sapi_TchH]	= L1SAP_COMMON_SAPI_TCH_H,
+	[GsmL1_Sapi_FacchH]	= L1SAP_COMMON_SAPI_FACCH_H,
+	[GsmL1_Sapi_Nch]	= L1SAP_COMMON_SAPI_NCH,
+	[GsmL1_Sapi_Pdtch]	= L1SAP_COMMON_SAPI_PDTCH,
+	[GsmL1_Sapi_Pacch]	= L1SAP_COMMON_SAPI_PACCH,
+	[GsmL1_Sapi_Pbcch]	= L1SAP_COMMON_SAPI_PBCCH,
+	[GsmL1_Sapi_Pagch]	= L1SAP_COMMON_SAPI_PAGCH,
+	[GsmL1_Sapi_Ppch]	= L1SAP_COMMON_SAPI_PPCH,
+	[GsmL1_Sapi_Pnch]	= L1SAP_COMMON_SAPI_PNCH,
+	[GsmL1_Sapi_Ptcch]	= L1SAP_COMMON_SAPI_PTCCH,
+	[GsmL1_Sapi_Prach]	= L1SAP_COMMON_SAPI_PRACH,
+};
+
+static enum l1sap_common_sapi get_common_sapi(GsmL1_Sapi_t sapi)
+{
+	if (sapi >= GsmL1_Sapi_NUM)
+		return L1SAP_COMMON_SAPI_UNKNOWN;
+	return common_sapi_by_sapi_t[sapi];
+}
+
+static void set_log_ctx_sapi(GsmL1_Sapi_t sapi)
+{
+	l1sap_log_ctx_sapi = get_common_sapi(sapi);
+	log_set_context(LOG_CTX_L1_SAPI, &l1sap_log_ctx_sapi);
+}
+
 static int handle_ph_readytosend_ind(struct femtol1_hdl *fl1,
 				     GsmL1_PhReadyToSendInd_t *rts_ind,
 				     struct msgb *l1p_msg)
@@ -800,6 +851,8 @@ static int handle_ph_readytosend_ind(struct femtol1_hdl *fl1,
 	uint8_t chan_nr, link_id;
 	uint32_t fn;
 
+	set_log_ctx_sapi(rts_ind->sapi);
+
 	/* check if primitive should be handled by common part */
 	chan_nr = chan_nr_by_sapi(&trx->ts[rts_ind->u8Tn], rts_ind->sapi,
 		rts_ind->subCh, rts_ind->u8Tn, rts_ind->u32Fn);
@@ -860,10 +913,6 @@ static int handle_ph_readytosend_ind(struct femtol1_hdl *fl1,
 	case GsmL1_Sapi_Prach:
 		goto empty_frame;
 		break;
-	case GsmL1_Sapi_Cbch:
-		/* get them from bts->si_buf[] */
-		bts_cbch_get(bts, msu_param->u8Buffer, &g_time);
-		break;
 	default:
 		memcpy(msu_param->u8Buffer, fill_frame, GSM_MACBLOCK_LEN);
 		break;
@@ -925,6 +974,8 @@ static int handle_ph_data_ind(struct femtol1_hdl *fl1, GsmL1_PhDataInd_t *data_i
 	struct gsm_time g_time;
 	int rc = 0;
 
+	set_log_ctx_sapi(data_ind->sapi);
+
 	chan_nr = chan_nr_by_sapi(&trx->ts[data_ind->u8Tn], data_ind->sapi,
 		data_ind->subCh, data_ind->u8Tn, data_ind->u32Fn);
 	if (!chan_nr) {
@@ -983,17 +1034,12 @@ static int handle_ph_ra_ind(struct femtol1_hdl *fl1, GsmL1_PhRaInd_t *ra_ind,
 			    struct msgb *l1p_msg)
 {
 	struct gsm_bts_trx *trx = femtol1_hdl_trx(fl1);
-	struct gsm_bts *bts = trx->bts;
 	struct gsm_lchan *lchan;
 	struct osmo_phsap_prim *l1sap;
 	int rc;
 	struct ph_rach_ind_param rach_ind_param;
 
-	/* FIXME: this should be deprecated/obsoleted as it bypasses rach.busy counting */
-	if (ra_ind->measParam.fLinkQuality < bts->min_qual_rach) {
-		msgb_free(l1p_msg);
-		return 0;
-	}
+	set_log_ctx_sapi(ra_ind->sapi);
 
 	dump_meas_res(LOGL_DEBUG, &ra_ind->measParam);
 
@@ -1016,13 +1062,14 @@ static int handle_ph_ra_ind(struct femtol1_hdl *fl1, GsmL1_PhRaInd_t *ra_ind,
 		.rssi = (int8_t) ra_ind->measParam.fRssi,
 		.ber10k = (unsigned int) (ra_ind->measParam.fBer * 10000.0),
 		.acc_delay_256bits = ra_ind->measParam.i16BurstTiming * 64,
+		.lqual_cb = (int16_t) ra_ind->measParam.fLinkQuality * 10, /* centiBels */
 	};
 
 	lchan = l1if_hLayer_to_lchan(trx, ra_ind->hLayer2);
 	if (!lchan || lchan->ts->pchan == GSM_PCHAN_CCCH ||
 	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4 ||
 	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH)
-		rach_ind_param.chan_nr = 0x88;
+		rach_ind_param.chan_nr = RSL_CHAN_RACH;
 	else
 		rach_ind_param.chan_nr = gsm_lchan2chan_nr(lchan);
 
@@ -1259,6 +1306,7 @@ int get_clk_cal(struct femtol1_hdl *hdl)
 #endif
 }
 
+#if SUPERFEMTO_API_VERSION >= SUPERFEMTO_API(2,2,0)
 /*
  * RevC was the last HW revision without an external
  * attenuator. Check for that.
@@ -1268,13 +1316,13 @@ static int has_external_atten(struct femtol1_hdl *hdl)
 	/* older version doesn't have an attenuator */
 	return hdl->hw_info.ver_major > 2;
 }
+#endif /* 2.2.0 */
 
 /* activate or de-activate the entire RF-Frontend */
 int l1if_activate_rf(struct femtol1_hdl *hdl, int on)
 {
 	struct msgb *msg = sysp_msgb_alloc();
 	SuperFemto_Prim_t *sysp = msgb_sysprim(msg);
-	struct gsm_bts_trx *trx = hdl->phy_inst->trx;
 
 	if (on) {
 		sysp->id = SuperFemto_PrimId_ActivateRfReq;
@@ -1302,6 +1350,7 @@ int l1if_activate_rf(struct femtol1_hdl *hdl, int on)
 		sysp->u.activateRfReq.rfRx.iClkCor = get_clk_cal(hdl);
 #endif /* API 2.4.0 */
 #if SUPERFEMTO_API_VERSION >= SUPERFEMTO_API(2,2,0)
+		struct gsm_bts_trx *trx = hdl->phy_inst->trx;
 		if (has_external_atten(hdl)) {
 			LOGP(DL1C, LOGL_INFO, "Using external attenuator.\n");
 			sysp->u.activateRfReq.rfTrx.u8UseExtAtten = 1;
@@ -1387,11 +1436,6 @@ static int mute_rf_compl_cb(struct gsm_bts_trx *trx, struct msgb *resp,
 /* mute/unmute RF time slots */
 int l1if_mute_rf(struct femtol1_hdl *hdl, uint8_t mute[8], l1if_compl_cb *cb)
 {
-	const uint8_t unmuted[8] = { 0,0,0,0,0,0,0,0 };
-	struct msgb *msg = sysp_msgb_alloc();
-	SuperFemto_Prim_t *sysp = msgb_sysprim(msg);
-	struct gsm_bts_trx *trx = hdl->phy_inst->trx;
-	int i;
 
 	LOGP(DL1C, LOGL_INFO, "Tx RF-MUTE.req (%d, %d, %d, %d, %d, %d, %d, %d)\n",
 	     mute[0], mute[1], mute[2], mute[3],
@@ -1399,18 +1443,22 @@ int l1if_mute_rf(struct femtol1_hdl *hdl, uint8_t mute[8], l1if_compl_cb *cb)
 	    );
 
 #if SUPERFEMTO_API_VERSION < SUPERFEMTO_API(3,6,0)
+	const uint8_t unmuted[8] = { 0,0,0,0,0,0,0,0 };
+	struct gsm_bts_trx *trx = hdl->phy_inst->trx;
+	int i;
 	LOGP(DL1C, LOGL_ERROR, "RF-MUTE.req not supported by SuperFemto\n");
-	msgb_free(msg);
 	/* always acknowledge an un-MUTE (which is a no-op if MUTE is not supported */
-	if (!memcmp(mute, unmuted, ARRAY_SIZE(mute))) {
+	if (!memcmp(mute, unmuted, ARRAY_SIZE(unmuted))) {
 		bts_update_status(BTS_STATUS_RF_MUTE, mute[0]);
 		oml_mo_rf_lock_chg(&trx->mo, mute, 1);
-		for (i = 0; i < ARRAY_SIZE(mute); ++i)
+		for (i = 0; i < ARRAY_SIZE(unmuted); ++i)
 			mute_handle_ts(&trx->ts[i], mute[i]);
 		return 0;
 	}
 	return -ENOTSUP;
 #else
+	struct msgb *msg = sysp_msgb_alloc();
+	SuperFemto_Prim_t *sysp = msgb_sysprim(msg);
 	sysp->id = SuperFemto_PrimId_MuteRfReq;
 	memcpy(sysp->u.muteRfReq.u8Mute, mute, sizeof(sysp->u.muteRfReq.u8Mute));
 	/* save for later use */
@@ -1427,7 +1475,6 @@ static int info_compl_cb(struct gsm_bts_trx *trx, struct msgb *resp,
 	SuperFemto_Prim_t *sysp = msgb_sysprim(resp);
 	SuperFemto_SystemInfoCnf_t *sic = &sysp->u.systemInfoCnf;
 	struct femtol1_hdl *fl1h = trx_femtol1_hdl(trx);
-	int rc;
 
 	fl1h->hw_info.dsp_version[0] = sic->dspVersion.major;
 	fl1h->hw_info.dsp_version[1] = sic->dspVersion.minor;
@@ -1470,7 +1517,7 @@ static int info_compl_cb(struct gsm_bts_trx *trx, struct msgb *resp,
 
 #if SUPERFEMTO_API_VERSION >= SUPERFEMTO_API(2,4,0)
 	/* load calibration tables (if we know their path) */
-	rc = calib_load(fl1h);
+	int rc = calib_load(fl1h);
 	if (rc < 0)
 		LOGP(DL1C, LOGL_ERROR, "Operating without calibration; "
 			"unable to load tables!\n");
@@ -1843,6 +1890,7 @@ static void fill_trx_power_params(struct gsm_bts_trx *trx,
 		/* sysmoBTS 1002 without any setting in EEPROM */
 		LOGP(DL1C, LOGL_NOTICE, "Assuming 1002 for sysmoBTS "
 			"Model number %u\n", fl1h->hw_info.model_nr);
+		/* fall-through */
 	case 1002:
 		set_power_param(&trx->power_params, 23, 0);
 	}
@@ -1875,7 +1923,7 @@ int bts_model_phy_link_open(struct phy_link *plink)
 			LOGP(DL1C, LOGL_NOTICE, "Cannot determine nominal "
 			     "transmit power. Assuming 23dBm.\n");
 		}
-		bts->c0->nominal_power = rc;
+		bts->c0->nominal_power = rc / 1000;
 	}
 
 	hdl = pinst->u.sysmobts.hdl;
