@@ -76,6 +76,7 @@ int bts_model_init(struct gsm_bts *bts)
 		exit(23);
 	}
 
+	gsm_bts_set_feature(bts, BTS_FEAT_CBCH);
 	gsm_bts_set_feature(bts, BTS_FEAT_GPRS);
 	gsm_bts_set_feature(bts, BTS_FEAT_EGPRS);
 	gsm_bts_set_feature(bts, BTS_FEAT_OML_ALERTS);
@@ -85,12 +86,18 @@ int bts_model_init(struct gsm_bts *bts)
 	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_F_EFR);
 	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_F_AMR);
 	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_H_AMR);
+	gsm_bts_set_feature(bts, BTS_FEAT_MS_PWR_CTRL_DSP);
 
 	bts_model_vty_init(bts);
 
 	return 0;
 }
 
+int bts_model_trx_init(struct gsm_bts_trx *trx)
+{
+	return 0;
+}
+
 int bts_model_oml_estab(struct gsm_bts *bts)
 {
 	return 0;
