@@ -270,7 +270,7 @@ int main(int argc, char **argv)
 		exit(2);
 
 	sysmobts_mgr_vty_init();
-	logging_vty_add_cmds(&mgr_log_info);
+	logging_vty_add_cmds();
 	rc = sysmobts_mgr_parse_config(&manager);
 	if (rc < 0) {
 		LOGP(DFIND, LOGL_FATAL, "Cannot parse config file\n");
@@ -312,7 +312,7 @@ int main(int argc, char **argv)
 		LOGP(DLCTRL, LOGL_ERROR, "Can't connect to CTRL @ localhost:%u\n",
 		     OSMO_CTRL_PORT_BTS);
 	else
-		LOGP(DLCTRL, LOGL_NOTICE, "CTRL connected to locahost:%u\n",
+		LOGP(DLCTRL, LOGL_NOTICE, "CTRL connected to localhost:%u\n",
 		     OSMO_CTRL_PORT_BTS);
 
         sysmobts_mgr_temp_init(&manager, ccon);
