@@ -25,9 +25,11 @@
 #include <stdlib.h>
 #include <stdbool.h>
 #include <errno.h>
+#include <inttypes.h>
 
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/bits.h>
+#include <osmocom/codec/ecu.h>
 #include <osmocom/gsm/abis_nm.h>
 
 #include <osmo-bts/logging.h>
@@ -48,14 +50,34 @@ static const uint8_t transceiver_chan_types[_GSM_PCHAN_MAX] = {
 	[GSM_PCHAN_NONE]                = 8,
 	[GSM_PCHAN_CCCH]                = 4,
 	[GSM_PCHAN_CCCH_SDCCH4]         = 5,
+	[GSM_PCHAN_CCCH_SDCCH4_CBCH]    = 5,
 	[GSM_PCHAN_TCH_F]               = 1,
 	[GSM_PCHAN_TCH_H]               = 3,
 	[GSM_PCHAN_SDCCH8_SACCH8C]      = 7,
+	[GSM_PCHAN_SDCCH8_SACCH8C_CBCH] = 7,
 	[GSM_PCHAN_PDCH]                = 13,
 	/* [GSM_PCHAN_TCH_F_PDCH] not needed here, see trx_set_ts_as_pchan() */
 	[GSM_PCHAN_UNKNOWN]             = 0,
 };
 
+static enum gsm_phys_chan_config transceiver_chan_type_2_pchan(uint8_t type)
+{
+	int i;
+	for (i = 0; i < _GSM_PCHAN_MAX; i++) {
+		if (transceiver_chan_types[i] == type)
+			return (enum gsm_phys_chan_config) i;
+	}
+	return GSM_PCHAN_UNKNOWN;
+}
+
+struct trx_l1h *trx_l1h_alloc(void *tall_ctx, struct phy_instance *pinst)
+{
+	struct trx_l1h *l1h;
+	l1h = talloc_zero(tall_ctx, struct trx_l1h);
+	l1h->phy_inst = pinst;
+	trx_if_init(l1h);
+	return l1h;
+}
 
 static void check_transceiver_availability_trx(struct trx_l1h *l1h, int avail)
 {
@@ -92,18 +114,6 @@ static void check_transceiver_availability_trx(struct trx_l1h *l1h, int avail)
 	}
 }
 
-int check_transceiver_availability(struct gsm_bts *bts, int avail)
-{
-	struct gsm_bts_trx *trx;
-
-	llist_for_each_entry(trx, &bts->trx_list, list) {
-		struct phy_instance *pinst = trx_phy_instance(trx);
-		struct trx_l1h *l1h = pinst->u.osmotrx.hdl;
-		check_transceiver_availability_trx(l1h, avail);
-	}
-	return 0;
-}
-
 int bts_model_lchan_deactivate(struct gsm_lchan *lchan)
 {
 	struct phy_instance *pinst = trx_phy_instance(lchan->ts->trx);
@@ -130,17 +140,75 @@ int bts_model_lchan_deactivate_sacch(struct gsm_lchan *lchan)
 				   LID_SACCH, 0);
 }
 
+static void l1if_setslot_cb(struct trx_l1h *l1h, uint8_t tn, uint8_t type, int rc)
+{
+	struct phy_instance *pinst = l1h->phy_inst;
+	struct gsm_bts_trx *trx = pinst->trx;
+	struct gsm_bts_trx_ts *ts;
+	enum gsm_phys_chan_config pchan;
+
+	if (tn >= TRX_NR_TS) {
+		LOGPPHI(pinst, DL1C, LOGL_ERROR, "transceiver SETSLOT invalid param TN (%" PRIu8 ")\n",
+			tn);
+		return;
+	}
+
+	pchan = transceiver_chan_type_2_pchan(type);
+	if (pchan == GSM_PCHAN_UNKNOWN) {
+		LOGPPHI(pinst, DL1C, LOGL_ERROR, "transceiver SETSLOT invalid param TS_TYPE (%" PRIu8 ")\n",
+			type);
+		return;
+	}
+
+	ts = &trx->ts[tn];
+	LOGPPHI(pinst, DL1C, LOGL_DEBUG, "%s l1if_setslot_cb(as_pchan=%s),"
+	     " calling cb_ts_connected(rc=%d)\n",
+	     gsm_ts_name(ts), gsm_pchan_name(pchan), rc);
+	cb_ts_connected(ts, rc);
+}
+
+static void l1if_poweronoff_cb(struct trx_l1h *l1h, bool poweronoff, int rc)
+{
+	struct phy_instance *pinst = l1h->phy_inst;
+	struct phy_link *plink = pinst->phy_link;
+
+	plink->u.osmotrx.powered = poweronoff;
+	plink->u.osmotrx.poweronoff_sent = false;
+
+	if (poweronoff) {
+		if (rc == 0 && pinst->phy_link->state != PHY_LINK_CONNECTED) {
+			trx_sched_clock_started(pinst->trx->bts);
+			phy_link_state_set(pinst->phy_link, PHY_LINK_CONNECTED);
+		} else if (rc != 0 && pinst->phy_link->state != PHY_LINK_SHUTDOWN) {
+			trx_sched_clock_stopped(pinst->trx->bts);
+			phy_link_state_set(pinst->phy_link, PHY_LINK_SHUTDOWN);
+		}
+	}
+}
+
 /*
  * transceiver provisioning
  */
 int l1if_provision_transceiver_trx(struct trx_l1h *l1h)
 {
 	uint8_t tn;
+	struct phy_instance *pinst = l1h->phy_inst;
+	struct phy_link *plink = pinst->phy_link;
 
-	if (!transceiver_available)
+	/* During setup, pinst may still not be associated to a TRX nr */
+	if (!pinst->trx) {
+		LOGPPHI(pinst, DL1C, LOGL_INFO,
+			"Delaying provision, TRX not yet assigned to phy instance\n");
 		return -EIO;
+	}
 
-	if (l1h->config.poweron
+	if (phy_link_state_get(plink) == PHY_LINK_SHUTDOWN) {
+		LOGPPHI(pinst, DL1C, LOGL_INFO,
+			"Delaying provision, TRX not yet available\n");
+		return -EIO;
+	}
+
+	if (l1h->config.enabled
 	 && l1h->config.tsc_valid
 	 && l1h->config.bsic_valid
 	 && l1h->config.arfcn_valid) {
@@ -159,9 +227,23 @@ int l1if_provision_transceiver_trx(struct trx_l1h *l1h)
 			l1h->config.bsic_sent = 1;
 		}
 
-		if (!l1h->config.poweron_sent) {
-			trx_if_cmd_poweron(l1h);
-			l1h->config.poweron_sent = 1;
+		/* Ask transceiver to use the newest TRXD header version if not using it yet */
+		if (!l1h->config.setformat_sent) {
+			if (l1h->config.trxd_hdr_ver_use != plink->u.osmotrx.trxd_hdr_ver_max) {
+				trx_if_cmd_setformat(l1h, plink->u.osmotrx.trxd_hdr_ver_max);
+				l1h->config.trxd_hdr_ver_req = plink->u.osmotrx.trxd_hdr_ver_max;
+			} else {
+				LOGPPHI(pinst, DL1C, LOGL_INFO,
+					"No need to negotiate TRXD version, "
+					"already using maximum configured one: %" PRIu8 "\n",
+					l1h->config.trxd_hdr_ver_use);
+			}
+			l1h->config.setformat_sent = 1;
+		}
+
+		if (pinst->num == 0 && !plink->u.osmotrx.powered && !plink->u.osmotrx.poweronoff_sent) {
+			trx_if_cmd_poweron(l1h, l1if_poweronoff_cb);
+			plink->u.osmotrx.poweronoff_sent = true;
 		}
 
 		/* after power on */
@@ -186,16 +268,18 @@ int l1if_provision_transceiver_trx(struct trx_l1h *l1h)
 			if (l1h->config.slottype_valid[tn]
 			 && !l1h->config.slottype_sent[tn]) {
 				trx_if_cmd_setslot(l1h, tn,
-					l1h->config.slottype[tn]);
+					l1h->config.slottype[tn], l1if_setslot_cb);
 				l1h->config.slottype_sent[tn] = 1;
 			}
 		}
 		return 0;
 	}
 
-	if (!l1h->config.poweron && !l1h->config.poweron_sent) {
-		trx_if_cmd_poweroff(l1h);
-		l1h->config.poweron_sent = 1;
+	if (!l1h->config.enabled) {
+		if (pinst->num == 0 && plink->u.osmotrx.powered && !plink->u.osmotrx.poweronoff_sent) {
+			trx_if_cmd_poweroff(l1h, l1if_poweronoff_cb);
+			plink->u.osmotrx.poweronoff_sent = true;
+		}
 		l1h->config.rxgain_sent = 0;
 		l1h->config.power_sent = 0;
 		l1h->config.maxdly_sent = 0;
@@ -215,10 +299,12 @@ int l1if_provision_transceiver(struct gsm_bts *bts)
 	llist_for_each_entry(trx, &bts->trx_list, list) {
 		struct phy_instance *pinst = trx_phy_instance(trx);
 		struct trx_l1h *l1h = pinst->u.osmotrx.hdl;
+		l1h->config.trxd_hdr_ver_req = 0;
+		l1h->config.trxd_hdr_ver_use = 0;
+		l1h->config.setformat_sent = 0;
 		l1h->config.arfcn_sent = 0;
 		l1h->config.tsc_sent = 0;
 		l1h->config.bsic_sent = 0;
-		l1h->config.poweron_sent = 0;
 		l1h->config.rxgain_sent = 0;
 		l1h->config.power_sent = 0;
 		l1h->config.maxdly_sent = 0;
@@ -241,9 +327,8 @@ static int trx_init(struct gsm_bts_trx *trx)
 	struct trx_l1h *l1h = pinst->u.osmotrx.hdl;
 
 	/* power on transceiver, if not already */
-	if (!l1h->config.poweron) {
-		l1h->config.poweron = 1;
-		l1h->config.poweron_sent = 0;
+	if (!l1h->config.enabled) {
+		l1h->config.enabled = true;
 		l1if_provision_transceiver_trx(l1h);
 	}
 
@@ -268,14 +353,14 @@ int bts_model_trx_close(struct gsm_bts_trx *trx)
 	trx_sched_reset(&l1h->l1s);
 
 	/* deactivate lchan for CCCH */
-	if (pchan == GSM_PCHAN_CCCH || pchan == GSM_PCHAN_CCCH_SDCCH4) {
+	if (pchan == GSM_PCHAN_CCCH || pchan == GSM_PCHAN_CCCH_SDCCH4 ||
+	    pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH) {
 		lchan_set_state(&trx->ts[0].lchan[CCCH_LCHAN], LCHAN_S_INACTIVE);
 	}
 
 	/* power off transceiver, if not already */
-	if (l1h->config.poweron) {
-		l1h->config.poweron = 0;
-		l1h->config.poweron_sent = 0;
+	if (l1h->config.enabled) {
+		l1h->config.enabled = false;
 		l1if_provision_transceiver_trx(l1h);
 	}
 
@@ -306,6 +391,7 @@ static uint8_t trx_set_bts(struct gsm_bts *bts, struct tlv_parsed *new_attr)
 
 	llist_for_each_entry(trx, &bts->trx_list, list) {
 		struct phy_instance *pinst = trx_phy_instance(trx);
+		struct phy_link *plink = pinst->phy_link;
 		struct trx_l1h *l1h = pinst->u.osmotrx.hdl;
 		if (l1h->config.bsic != bsic || !l1h->config.bsic_valid) {
 			l1h->config.bsic = bsic;
@@ -313,9 +399,8 @@ static uint8_t trx_set_bts(struct gsm_bts *bts, struct tlv_parsed *new_attr)
 			l1h->config.bsic_sent = 0;
 			l1if_provision_transceiver_trx(l1h);
 		}
+		check_transceiver_availability_trx(l1h, phy_link_state_get(plink) != PHY_LINK_SHUTDOWN);
 	}
-	check_transceiver_availability(bts, transceiver_available);
-
 
 	return 0;
 }
@@ -378,13 +463,14 @@ static uint8_t trx_set_ts_as_pchan(struct gsm_bts_trx_ts *ts,
 		return NM_NACK_RES_NOTAVAIL;
 
 	/* activate lchan for CCCH */
-	if (pchan == GSM_PCHAN_CCCH || pchan == GSM_PCHAN_CCCH_SDCCH4) {
+	if (pchan == GSM_PCHAN_CCCH || pchan == GSM_PCHAN_CCCH_SDCCH4 ||
+	    pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH) {
 		ts->lchan[CCCH_LCHAN].rel_act_kind = LCHAN_REL_ACT_OML;
 		lchan_set_state(&ts->lchan[CCCH_LCHAN], LCHAN_S_ACTIVE);
 	}
 
 	slottype = transceiver_chan_types[pchan];
-	
+
 	if (l1h->config.slottype[tn] != slottype
 	 || !l1h->config.slottype_valid[tn]) {
 		l1h->config.slottype[tn] = slottype;
@@ -509,7 +595,7 @@ int l1if_process_meas_res(struct gsm_bts_trx *trx, uint8_t tn, uint32_t fn, uint
 	/* 100% BER is n_bits_total is 0 */
 	float ber = n_bits_total==0 ? 1.0 : (float)n_errors / (float)n_bits_total;
 
-	LOGPFN(DMEAS, LOGL_DEBUG, fn, "RX L1 frame %s fn=%u chan_nr=0x%02x MS pwr=%ddBm rssi=%.1f dBFS "
+	LOGPFN(DMEAS, LOGL_DEBUG, fn, "RX UL measurement for %s fn=%u chan_nr=0x%02x MS pwr=%ddBm rssi=%.1f dBFS "
 		"ber=%.2f%% (%d/%d bits) L1_ta=%d rqd_ta=%d toa256=%d\n",
 		gsm_lchan_name(lchan), fn, chan_nr, ms_pwr_dbm(lchan->ts->trx->bts->band, lchan->ms_power_ctrl.current),
 		rssi, ber*100, n_errors, n_bits_total, lchan->meas.l1_info[1], lchan->rqd_ta, toa256);
@@ -562,6 +648,23 @@ int bts_model_l1sap_down(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 						" chan_nr 0x%02x\n", chan_nr);
 					break;
 				}
+
+				/* attempt to allocate an Error Concealment Unit instance, if available */
+				lchan->ecu_state = osmo_ecu_init(trx, lchan2ecu_codec(lchan));
+
+				/* trx_chan_desc[] in scheduler.c uses the RSL_CHAN_OSMO_PDCH cbits
+				 * (0xc0) to indicate the need for PDTCH and PTCCH SAPI activation.
+				 * However, 0xc0 is a cbits pattern exclusively used for Osmocom style
+				 * dyn TS (a non-standard RSL Chan Activ mod); hence, for IPA style dyn
+				 * TS, the chan_nr will never reflect 0xc0 and we would omit the
+				 * PDTCH,PTTCH SAPIs. To properly de-/activate the PDTCH SAPIs in
+				 * scheduler.c, make sure the 0xc0 cbits are set for de-/activating PDTCH
+				 * lchans, i.e. both Osmocom and IPA style dyn TS. (For Osmocom style dyn
+				 * TS, the chan_nr typically already reflects 0xc0, while it doesn't for
+				 * IPA style.) */
+				if (lchan->type == GSM_LCHAN_PDTCH)
+					chan_nr = RSL_CHAN_OSMO_PDCH | (chan_nr & ~RSL_CHAN_NR_MASK);
+
 				/* activate dedicated channel */
 				trx_sched_set_lchan(&l1h->l1s, chan_nr, LID_DEDIC, 1);
 				/* activate associated channel */
@@ -594,6 +697,10 @@ int bts_model_l1sap_down(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 				break;
 			}
 			if (l1sap->u.info.type == PRIM_INFO_MODIFY) {
+				/* ECU for possibly new codec */
+				if (lchan->ecu_state)
+					osmo_ecu_destroy(lchan->ecu_state);
+				lchan->ecu_state = osmo_ecu_init(trx, lchan2ecu_codec(lchan));
 				/* change mode */
 				trx_sched_set_mode(&l1h->l1s, chan_nr,
 					lchan->rsl_cmode, lchan->tch_mode,
@@ -612,6 +719,11 @@ int bts_model_l1sap_down(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 					"chan_nr 0x%02x\n", chan_nr);
 				break;
 			}
+			/* clear ECU state (if any) */
+			if (lchan->ecu_state) {
+				osmo_ecu_destroy(lchan->ecu_state);
+				lchan->ecu_state = NULL;
+			}
 			/* deactivate associated channel */
 			bts_model_lchan_deactivate_sacch(lchan);
 			if (!l1sap->u.info.u.act_req.sacch_only) {
@@ -738,7 +850,7 @@ int bts_model_ts_disconnect(struct gsm_bts_trx_ts *ts)
 	return 0;
 }
 
-int bts_model_ts_connect(struct gsm_bts_trx_ts *ts,
+void bts_model_ts_connect(struct gsm_bts_trx_ts *ts,
 			 enum gsm_phys_chan_config as_pchan)
 {
 	int rc;
@@ -747,12 +859,7 @@ int bts_model_ts_connect(struct gsm_bts_trx_ts *ts,
 
 	rc = trx_set_ts_as_pchan(ts, as_pchan);
 	if (rc)
-		return rc;
+		cb_ts_connected(ts, rc);
 
-	LOGP(DL1C, LOGL_NOTICE, "%s bts_model_ts_connect(as_pchan=%s) success,"
-	     " calling cb_ts_connected()\n",
-	     gsm_ts_name(ts), gsm_pchan_name(as_pchan));
-
-	cb_ts_connected(ts);
-	return 0;
+	/* cb_ts_connected will be called in l1if_setslot_cb once we receive RSP SETSLOT */
 }
