@@ -97,6 +97,10 @@ int bts_model_handle_options(int argc, char **argv)
 
 int bts_model_init(struct gsm_bts *bts)
 {
+	struct bts_trx_priv *bts_trx = talloc_zero(bts, struct bts_trx_priv);
+	bts_trx->clk_s.fn_timer_ofd.fd = -1;
+
+	bts->model_priv = bts_trx;
 	bts->variant = BTS_OSMO_TRX;
 	bts->support.ciphers = CIPHER_A5(1) | CIPHER_A5(2) | CIPHER_A5(3);
 
@@ -111,12 +115,18 @@ int bts_model_init(struct gsm_bts *bts)
 	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_F_EFR);
 	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_F_AMR);
 	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_H_AMR);
+	gsm_bts_set_feature(bts, BTS_FEAT_CBCH);
 
 	bts_model_vty_init(bts);
 
 	return 0;
 }
 
+int bts_model_trx_init(struct gsm_bts_trx *trx)
+{
+	return 0;
+}
+
 void bts_model_phy_link_set_defaults(struct phy_link *plink)
 {
 	plink->u.osmotrx.local_ip = talloc_strdup(plink, "127.0.0.1");
@@ -126,15 +136,14 @@ void bts_model_phy_link_set_defaults(struct phy_link *plink)
 	plink->u.osmotrx.clock_advance = 20;
 	plink->u.osmotrx.rts_advance = 5;
 	plink->u.osmotrx.trx_ta_loop = true;
-	plink->u.osmotrx.trx_ms_power_loop = false;
-	plink->u.osmotrx.trx_target_rssi = -10;
+	/* attempt use newest TRXD version by default: */
+	plink->u.osmotrx.trxd_hdr_ver_max = TRX_DATA_FORMAT_VER;
 }
 
 void bts_model_phy_instance_set_defaults(struct phy_instance *pinst)
 {
 	struct trx_l1h *l1h;
-	l1h = talloc_zero(tall_bts_ctx, struct trx_l1h);
-	l1h->phy_inst = pinst;
+	l1h = trx_l1h_alloc(tall_bts_ctx, pinst);
 	pinst->u.osmotrx.hdl = l1h;
 
 	l1h->config.power_oml = 1;
