@@ -32,6 +32,7 @@
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/select.h>
 #include <osmocom/core/bits.h>
+#include <osmocom/core/socket.h>
 
 #include <osmocom/vty/vty.h>
 #include <osmocom/vty/command.h>
@@ -57,19 +58,18 @@ DEFUN(show_transceiver, show_transceiver_cmd, "show transceiver",
 	struct gsm_bts_trx *trx;
 	struct trx_l1h *l1h;
 
-	if (!transceiver_available) {
-		vty_out(vty, "transceiver is not connected%s", VTY_NEWLINE);
-	} else {
-		vty_out(vty, "transceiver is connected%s", VTY_NEWLINE);
-	}
-
 	llist_for_each_entry(trx, &bts->trx_list, list) {
 		struct phy_instance *pinst = trx_phy_instance(trx);
+		struct phy_link *plink = pinst->phy_link;
+		char *sname = osmo_sock_get_name(NULL, plink->u.osmotrx.trx_ofd_clk.fd);
 		l1h = pinst->u.osmotrx.hdl;
-		vty_out(vty, "TRX %d%s", trx->nr, VTY_NEWLINE);
+		vty_out(vty, "TRX %d %s%s", trx->nr, sname, VTY_NEWLINE);
+		talloc_free(sname);
 		vty_out(vty, " %s%s",
-			(l1h->config.poweron) ? "poweron":"poweroff",
+			trx_if_powered(l1h) ? "poweron":"poweroff",
 			VTY_NEWLINE);
+		vty_out(vty, "phy link state: %s%s",
+			phy_link_state_name(phy_link_state_get(plink)), VTY_NEWLINE);
 		if (l1h->config.arfcn_valid)
 			vty_out(vty, " arfcn  : %d%s%s",
 				(l1h->config.arfcn & ~ARFCN_PCS),
@@ -160,26 +160,23 @@ DEFUN(show_phy, show_phy_cmd, "show phy",
 	return CMD_SUCCESS;
 }
 
-DEFUN(cfg_phy_ms_power_loop, cfg_phy_ms_power_loop_cmd,
+DEFUN_DEPRECATED(cfg_phy_ms_power_loop, cfg_phy_ms_power_loop_cmd,
 	"osmotrx ms-power-loop <-127-127>", OSMOTRX_STR
 	"Enable MS power control loop\nTarget RSSI value (transceiver specific, "
 	"should be 6dB or more above noise floor)\n")
 {
-	struct phy_link *plink = vty->index;
+	vty_out (vty, "'osmotrx ms-power-loop' is deprecated, use 'uplink-power-target' instead%s", VTY_NEWLINE);
 
-	plink->u.osmotrx.trx_target_rssi = atoi(argv[0]);
-	plink->u.osmotrx.trx_ms_power_loop = true;
+	vty_bts->ul_power_target = atoi(argv[0]);
 
 	return CMD_SUCCESS;
 }
 
-DEFUN(cfg_phy_no_ms_power_loop, cfg_phy_no_ms_power_loop_cmd,
+DEFUN_DEPRECATED(cfg_phy_no_ms_power_loop, cfg_phy_no_ms_power_loop_cmd,
 	"no osmotrx ms-power-loop",
 	NO_STR OSMOTRX_STR "Disable MS power control loop\n")
 {
-	struct phy_link *plink = vty->index;
-
-	plink->u.osmotrx.trx_ms_power_loop = false;
+	vty_out (vty, "'no osmotrx ms-power-loop' is deprecated, use of BTS control loop is managed by BSC%s", VTY_NEWLINE);
 
 	return CMD_SUCCESS;
 }
@@ -234,7 +231,7 @@ DEFUN(cfg_phyinst_maxdlynb, cfg_phyinst_maxdlynb_cmd,
 	OSMOTRX_STR
 	"Set the maximum acceptable delay of a Normal Burst (in GSM symbols)."
 	" USE FOR TESTING ONLY, DON'T CHANGE IN PRODUCTION USE!"
-	" During normal operation, Normal Bursts delay are controled by a Timing"
+	" During normal operation, Normal Bursts delay are controlled by a Timing"
 	" Advance control loop and thus Normal Bursts arrive to a BTS with no more"
 	" than a couple GSM symbols, which is already taken into account in osmo-trx."
 	" So changing this setting will have no effect in production installations"
@@ -284,9 +281,9 @@ DEFUN(cfg_phyinst_power_on, cfg_phyinst_power_on_cmd,
 	struct trx_l1h *l1h = pinst->u.osmotrx.hdl;
 
 	if (strcmp(argv[0], "on"))
-		vty_out(vty, "OFF: %d%s", trx_if_cmd_poweroff(l1h), VTY_NEWLINE);
+		vty_out(vty, "OFF: %d%s", trx_if_cmd_poweroff(l1h, NULL), VTY_NEWLINE);
 	else {
-		vty_out(vty, "ON: %d%s", trx_if_cmd_poweron(l1h), VTY_NEWLINE);
+		vty_out(vty, "ON: %d%s", trx_if_cmd_poweron(l1h, NULL), VTY_NEWLINE);
 	}
 
 	return CMD_SUCCESS;
@@ -493,6 +490,29 @@ DEFUN(cfg_phy_no_setbsic, cfg_phy_no_setbsic_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN(cfg_phy_trxd_max_version, cfg_phy_trxd_max_version_cmd,
+	"osmotrx trxd-max-version (latest|<0-15>)", OSMOTRX_STR
+	"Set maximum TRXD format version to negotiate with TRX\n"
+	"Use latest supported TRXD format version (default)\n"
+	"Maximum TRXD format version number\n")
+{
+	struct phy_link *plink = vty->index;
+
+	int max_ver;
+	if (strcmp(argv[0], "latest") == 0)
+		max_ver = TRX_DATA_FORMAT_VER;
+	else
+		max_ver = atoi(argv[0]);
+	if (max_ver > TRX_DATA_FORMAT_VER) {
+		vty_out(vty, "%% Format version %d is not supported, maximum supported is %d%s",
+			max_ver, TRX_DATA_FORMAT_VER, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+	plink->u.osmotrx.trxd_hdr_ver_max = max_ver;
+
+	return CMD_SUCCESS;
+}
+
 void bts_model_config_write_phy(struct vty *vty, struct phy_link *plink)
 {
 	if (plink->u.osmotrx.local_ip)
@@ -502,10 +522,6 @@ void bts_model_config_write_phy(struct vty *vty, struct phy_link *plink)
 		vty_out(vty, " osmotrx ip remote %s%s",
 			plink->u.osmotrx.remote_ip, VTY_NEWLINE);
 
-	if (plink->u.osmotrx.trx_ms_power_loop)
-		vty_out(vty, " osmotrx ms-power-loop %d%s", plink->u.osmotrx.trx_target_rssi, VTY_NEWLINE);
-	else
-		vty_out(vty, " no osmotrx ms-power-loop%s", VTY_NEWLINE);
 	vty_out(vty, " %sosmotrx timing-advance-loop%s", (plink->u.osmotrx.trx_ta_loop) ? "" : "no ", VTY_NEWLINE);
 
 	if (plink->u.osmotrx.base_port_local)
@@ -522,6 +538,9 @@ void bts_model_config_write_phy(struct vty *vty, struct phy_link *plink)
 
 	if (plink->u.osmotrx.use_legacy_setbsic)
 		vty_out(vty, " osmotrx legacy-setbsic%s", VTY_NEWLINE);
+
+	if (plink->u.osmotrx.trxd_hdr_ver_max != TRX_DATA_FORMAT_VER)
+		vty_out(vty, " osmotrx trxd-max-version %d%s", plink->u.osmotrx.trxd_hdr_ver_max, VTY_NEWLINE);
 }
 
 void bts_model_config_write_phy_inst(struct vty *vty, struct phy_instance *pinst)
@@ -581,6 +600,7 @@ int bts_model_vty_init(struct gsm_bts *bts)
 	install_element(PHY_NODE, &cfg_phy_osmotrx_ip_cmd);
 	install_element(PHY_NODE, &cfg_phy_setbsic_cmd);
 	install_element(PHY_NODE, &cfg_phy_no_setbsic_cmd);
+	install_element(PHY_NODE, &cfg_phy_trxd_max_version_cmd);
 
 	install_element(PHY_INST_NODE, &cfg_phyinst_rxgain_cmd);
 	install_element(PHY_INST_NODE, &cfg_phyinst_tx_atten_cmd);
