@@ -121,9 +121,10 @@ static void virt_um_rcv_cb(struct virt_um_inst *vui, struct msgb *msg)
 		l1sap.u.rach_ind.acc_delay = 0; /* probably not used in virt um */
 		l1sap.u.rach_ind.is_11bit = 0;
 		l1sap.u.rach_ind.fn = fn;
-		/* we don't rally know which RACH bursrt type the virtual MS is using, as this field is not
+		/* we don't really know which RACH burst type the virtual MS is using, as this field is not
 		 * part of information present in the GSMTAP header.  So we simply report all of them as 0 */
 		l1sap.u.rach_ind.burst_type = GSM_L1_BURST_TYPE_ACCESS_0;
+		l1sap.u.rach_ind.lqual_cb = 10 * signal_dbm; /* Link quality in centiBel = 10 * dB. */
 		break;
 	case GSMTAP_CHANNEL_TCH_F:
 	case GSMTAP_CHANNEL_TCH_H:
@@ -207,7 +208,7 @@ int bts_model_phy_link_open(struct phy_link *plink)
 	llist_for_each_entry(pinst, &plink->instances, list) {
 		trx_sched_init(&pinst->u.virt.sched, pinst->trx);
 		/* Only start the scheduler for the transceiver on C0.
-		 * If we have multiple tranceivers, CCCH is always on C0
+		 * If we have multiple transceivers, CCCH is always on C0
 		 * and has to be auto active */
 		/* Other TRX are activated via OML by a PRIM_INFO_MODIFY
 		 * / PRIM_INFO_ACTIVATE */
@@ -319,7 +320,7 @@ static int l1if_process_meas_res(struct gsm_bts_trx *trx, uint8_t tn, uint32_t f
 
 	DEBUGPFN(DMEAS, fn, "RX L1 frame %s chan_nr=0x%02x MS pwr=%ddBm rssi=%.1f dBFS "
 		"ber=%.2f%% (%d/%d bits) L1_ta=%d rqd_ta=%d toa=%.2f\n",
-		gsm_lchan_name(lchan), chan_nr, ms_pwr_dbm(lchan->ts->trx->bts->band, lchan->ms_power),
+		gsm_lchan_name(lchan), chan_nr, ms_pwr_dbm(lchan->ts->trx->bts->band, lchan->ms_power_ctrl.max),
 		rssi, ber*100, n_errors, n_bits_total, lchan->meas.l1_info[1], lchan->rqd_ta, toa);
 
 	l1if_fill_meas_res(&l1sap, chan_nr, lchan->rqd_ta + toa, ber, rssi, fn);
