@@ -47,6 +47,7 @@
 #include <osmo-bts/l1sap.h>
 #include <osmo-bts/phy_link.h>
 #include "virtual_um.h"
+#include "l1_if.h"
 
 /* dummy, since no direct dsp support */
 uint32_t trx_get_hlayer1(struct gsm_bts_trx *trx)
@@ -56,6 +57,8 @@ uint32_t trx_get_hlayer1(struct gsm_bts_trx *trx)
 
 int bts_model_init(struct gsm_bts *bts)
 {
+	struct bts_virt_priv *bts_virt = talloc_zero(bts, struct bts_virt_priv);
+	bts->model_priv = bts_virt;
 	bts->variant = BTS_OSMO_VIRTUAL;
 	bts->support.ciphers = CIPHER_A5(1) | CIPHER_A5(2) | CIPHER_A5(3);
 
@@ -65,12 +68,18 @@ int bts_model_init(struct gsm_bts *bts)
 	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_F_EFR);
 	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_F_AMR);
 	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_H_AMR);
+	gsm_bts_set_feature(bts, BTS_FEAT_CBCH);
 
 	bts_model_vty_init(bts);
 
 	return 0;
 }
 
+int bts_model_trx_init(struct gsm_bts_trx *trx)
+{
+	return 0;
+}
+
 void bts_model_print_help()
 {
 	LOGP(DSUM, LOGL_NOTICE, "Unimplemented %s\n", __func__);
@@ -127,10 +136,9 @@ int bts_model_ts_disconnect(struct gsm_bts_trx_ts *ts)
 	return -ENOTSUP;
 }
 
-int bts_model_ts_connect(struct gsm_bts_trx_ts *ts, enum gsm_phys_chan_config as_pchan)
+void bts_model_ts_connect(struct gsm_bts_trx_ts *ts, enum gsm_phys_chan_config as_pchan)
 {
 	LOGP(DSUM, LOGL_NOTICE, "Unimplemented %s\n", __func__);
-	return -ENOTSUP;
 }
 
 int main(int argc, char **argv)
