@@ -1,4 +1,4 @@
-/* Scheduler worker functiosn for Virtua OsmoBTS */
+/* Scheduler worker functions for Virtua OsmoBTS */
 
 /* (C) 2015-2017 by Harald Welte <laforge@gnumonks.org>
  * (C) 2017 Sebastian Stumpf <sebastian.stumpf87@googlemail.com>
@@ -64,7 +64,7 @@ static void tx_to_virt_um(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 {
 	const struct trx_chan_desc *chdesc = &trx_chan_desc[chan];
 	struct msgb *outmsg;			/* msg to send with gsmtap header prepended */
-	uint16_t arfcn = l1t->trx->arfcn;	/* ARFCN of the tranceiver the message is send with */
+	uint16_t arfcn = l1t->trx->arfcn;	/* ARFCN of the transceiver the message is send with */
 	uint8_t signal_dbm = 63;		/* signal strength, 63 is best */
 	uint8_t snr = 63;			/* signal noise ratio, 63 is best */
 	uint8_t *data = msgb_l2(msg);		/* data to transmit (whole message without l1 header) */
@@ -79,7 +79,8 @@ static void tx_to_virt_um(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	timeslot = tn;
 	/* in Osmocom, AGCH is only sent on ccch block 0. no idea why. this seems to cause false GSMTAP channel
 	 * types for agch and pch. */
-	if (rsl_chantype == RSL_CHAN_PCH_AGCH && L1SAP_FN2CCCHBLOCK(fn) == 0)
+	if (rsl_chantype == RSL_CHAN_PCH_AGCH &&
+	    l1sap_fn2ccch_block(fn) >= num_agch(l1t->trx, "PH-DATA-REQ"))
 		gsmtap_chantype = GSMTAP_CHANNEL_PCH;
 	else
 		gsmtap_chantype = chantype_rsl2gsmtap(rsl_chantype, chdesc->link_id); /* the logical channel type */
@@ -198,7 +199,7 @@ static void tx_tch_common(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 #if 0
 	/* handle loss detection of received TCH frames */
 	if (rsl_cmode == RSL_CMOD_SPD_SPEECH
-	 && ++(chan_state->lost) > 5) {
+	 && ++(chan_state->lost_frames) > 5) {
 		uint8_t tch_data[GSM_FR_BYTES];
 		int len;
 
@@ -462,7 +463,7 @@ ubit_t *tx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	if (msg_facch) {
 		tx_to_virt_um(l1t, tn, fn, chan, msg_facch);
 		msgb_free(msg_tch);
-	} else
+	} else if (msg_tch)
 		tx_to_virt_um(l1t, tn, fn, chan, msg_tch);
 
 send_burst:
@@ -478,38 +479,33 @@ send_burst:
  * directly into the L1SAP, bypassing the TDMA multiplex logic oriented
  * towards receiving bursts */
 
-int rx_rach_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, int16_t toa256)
+int rx_rach_fn(struct l1sched_trx *l1t, enum trx_chan_type chan,
+	       uint8_t bid, const struct trx_ul_burst_ind *bi)
 {
 	return 0;
 }
 
 /*! \brief a single burst was received by the PHY, process it */
-int rx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, int16_t toa256)
+int rx_data_fn(struct l1sched_trx *l1t, enum trx_chan_type chan,
+	       uint8_t bid, const struct trx_ul_burst_ind *bi)
 {
 	return 0;
 }
 
-int rx_pdtch_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, int16_t toa256)
+int rx_pdtch_fn(struct l1sched_trx *l1t, enum trx_chan_type chan,
+	        uint8_t bid, const struct trx_ul_burst_ind *bi)
 {
 	return 0;
 }
 
-int rx_tchf_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, int16_t toa256)
+int rx_tchf_fn(struct l1sched_trx *l1t, enum trx_chan_type chan,
+	       uint8_t bid, const struct trx_ul_burst_ind *bi)
 {
 	return 0;
 }
 
-int rx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
-	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, int16_t toa256)
+int rx_tchh_fn(struct l1sched_trx *l1t, enum trx_chan_type chan,
+	       uint8_t bid, const struct trx_ul_burst_ind *bi)
 {
 	return 0;
 }
@@ -566,8 +562,9 @@ static int vbts_sched_fn(struct gsm_bts *bts, uint32_t fn)
 static void vbts_fn_timer_cb(void *data)
 {
 	struct gsm_bts *bts = data;
+	struct bts_virt_priv *bts_virt = (struct bts_virt_priv *)bts->model_priv;
 	struct timeval tv_now;
-	struct timeval *tv_clock = &bts->vbts.tv_clock;
+	struct timeval *tv_clock = &bts_virt->tv_clock;
 	int32_t elapsed_us;
 
 	gettimeofday(&tv_now, NULL);
@@ -591,28 +588,29 @@ static void vbts_fn_timer_cb(void *data)
 		};
 		timeradd(tv_clock, &tv_frame, tv_clock);
 		/* increment the frame number in the BTS model instance */
-		bts->vbts.last_fn = (bts->vbts.last_fn + 1) % GSM_HYPERFRAME;
-		vbts_sched_fn(bts, bts->vbts.last_fn);
+		bts_virt->last_fn = (bts_virt->last_fn + 1) % GSM_HYPERFRAME;
+		vbts_sched_fn(bts, bts_virt->last_fn);
 		elapsed_us -= FRAME_DURATION_uS;
 	}
 
 	/* re-schedule the timer */
 	/* timer is set to frame duration - elapsed time to guarantee that this cb method will be
 	 * periodically executed every 4.615ms */
-	osmo_timer_schedule(&bts->vbts.fn_timer, 0, FRAME_DURATION_uS - elapsed_us);
+	osmo_timer_schedule(&bts_virt->fn_timer, 0, FRAME_DURATION_uS - elapsed_us);
 }
 
 int vbts_sched_start(struct gsm_bts *bts)
 {
+	struct bts_virt_priv *bts_virt = (struct bts_virt_priv *)bts->model_priv;
 	LOGP(DL1P, LOGL_NOTICE, "starting VBTS scheduler\n");
 
-	memset(&bts->vbts.fn_timer, 0, sizeof(bts->vbts.fn_timer));
-	bts->vbts.fn_timer.cb = vbts_fn_timer_cb;
-	bts->vbts.fn_timer.data = bts;
+	memset(&bts_virt->fn_timer, 0, sizeof(bts_virt->fn_timer));
+	bts_virt->fn_timer.cb = vbts_fn_timer_cb;
+	bts_virt->fn_timer.data = bts;
 
-	gettimeofday(&bts->vbts.tv_clock, NULL);
+	gettimeofday(&bts_virt->tv_clock, NULL);
 	/* trigger the first timer after 4615us (a frame duration) */
-	osmo_timer_schedule(&bts->vbts.fn_timer, 0, FRAME_DURATION_uS);
+	osmo_timer_schedule(&bts_virt->fn_timer, 0, FRAME_DURATION_uS);
 
 	return 0;
 }
